	PROGRAM EMITSUM
C
C	TOTAL PROFILE EMISSIONS SUMMATION PROGRAM
C
C	The EmitSum program is used to produce a compound
C	composition profile (.EMC) file giving total emissions of
C	compounds into a model domain, given total emissions of
C	profiles into the domain and the compounds in each of the
C	profiles.
C
C	To use the program, use one of the following commands:
C
C	EMITSUM InputParmFile
C	EMITSUM FILES=EmissionsSummaryFile OutputProfile ProfileLstFile
C
C	In the first method, InputParmFile is the name of a file that
C	contains the parameters and file name used.
C	In the second method, the emissions summary input and output
C	file and the profile listing file names are given on the
C	command line.  It is equivalent to running the program with 
C	the following parameters in the input parameter file.
C		EMITSUM-IN=EmissionsSummaryFile
C		EMITSUM=OutputProfile
C		PROFLIST=ProfileLstFile
C	The parameters not given in the input parameter file or the
C	command line are taken from MODELING.PRM.
C
C
C	PROGRAM PARAMETERS
C
	PARAMETER (MAXCMT=10)
	CHARACTER INFILE*60,EMITSM*8,EMCUIC*60,CMNT(MAXCMT)*250
	CHARACTER PRLIST*60,ESMLOC*60,DESC*250,OUTFIL*80,EDBUIC*60
	CHARACTER EMCFIL*60,SUMIN*60,PRMFIL*80,LOGFIL*60
	INTEGER CMTTYP(MAXCMT),NCMT
C
C	PROFILES
C
	PARAMETER (MAXPRO=1000)
	CHARACTER PRONUM(MAXPRO)*4,PRONAM(MAXPRO)*8,PROTYP*4
	INTEGER NPRO
	DATA PROTYP/'TOG '/
C
C	COMPOUND NAMES AND AMOUNTS
C
	PARAMETER (MAXSPC=2000)
	CHARACTER SPCNAM(MAXSPC)*16,SPCDSC(MAXSPC)*255,
     &	 SPCVOL(MAXSPC)*1,SPCXMT(MAXSPC)*1
	REAL SPCMAS(MAXSPC)
	INTEGER NSPC,SPCIND(MAXSPC)
C
C	LOCAL VARIABLES
C
	CHARACTER INLIN*255,CNAME*16,PNUM*4,PTYP*4,NAMEIN*8
	LOGICAL ERRIND,MPIN
C
C
C	OPEN INPUT FILE AND READ PARAMETERS
C
	CALL GETARG(1,PRMFIL)			! GET COMMAND LINE
	IF (PRMFIL(1:1).EQ.' ') THEN
		WRITE (6,*) 'Emissions profile summation program'
		WRITE (6,*) 'Use: EMITSUM parameterfile'
		WRITE (6,*) 'Or:  EMITSUM '
     &		 //'FILES=EmissionsSummaryInputFile '
     &		 //'EmissionsSummrayProfile'
		CALL EXIT (2)
	ENDIF
C
	NCMT=0
	INFILE=' '
	EMITSM=' '
	ESMLOC=' '
	EMCUIC=' '
	PRLIST=' '
	EDBUIC=' '
	LOGFIL='EMITSUM.CSV'
	LLOGFL=11
	DESC=' '
C
C	FIRST GET PARMS FROM MODELING.PRM, THEN FROM INPUT FILE
C
	MPIN=.TRUE.
	OPEN (UNIT=1,FILE='MODELING.PRM',STATUS='UNKNOWN')
C
  10	READ (1,1,END=12) INLIN
    1	FORMAT (A)
C
C	EMISSIONS PROFILE TOTALS OUTPUT FILE (REQUIRED)
C
	IF (INLIN(1:8).EQ.'EMITSUM=') THEN
		EMITSM=INLIN(9:)
		LEMTSM=LNBLNK(EMITSM)
C
C	EMISSIONS PROFILE TOTALS INPUT FILE (REQUIRED)
C
	ELSEIF (INLIN(1:11).EQ.'EMITSUM-IN=') THEN
		SUMIN=INLIN(12:)
		LSUMIN=LNBLNK(SUMIN)
C
C	PROFILE LIST (REQUIRED)
C
	ELSEIF (INLIN(1:9).EQ.'PROFLIST=') THEN
		PRLIST=INLIN(10:)
		LPRLST=LNBLNK(PRLIST)
C
C	LOCATION OF INPUT .EMC FILES THAT GIVE PROFILE COMPOSITIONS
C	IN TERMS OF CHEMICAL COMPOUNDS USING SPECIATION DB DESIGNATIONS
C	ALSO DEFAULT LOCATION FOR OUTPUT FILE
C
	ELSEIF (INLIN(1:8).EQ.'PROFEMC=') THEN
		EMCUIC=INLIN(9:)
		LEMCUC=LNBLNK(EMCUIC)
C
C	LOCATION OF OUTPUT TOTAL EMISSIONS .EMC FILE IF NOT PROFMEC
C
	ELSEIF (INLIN(1:12).EQ.'EMITSUM-LOC=') THEN
		ESMLOC=INLIN(13:)
		LESMLC=LNBLNK(ESMLOC)
C
C	LOCATION OF EMISSIONS DB FILES TO GET CMPDINFO.DAT
C
	ELSEIF (INLIN(1:7).EQ.'EMITDB=') THEN
		EDBUIC=INLIN(8:)
		LEDBUC=LNBLNK(EDBUIC)
C
C	DESCRIPTION, COMMENTS OR NOTES FOR OUTPUT FILE
C
	ELSEIF (INLIN(1:13).EQ.'EMITSUM-DESC=') THEN
		DESC=INLIN(14:)
	ELSEIF (INLIN(1:12).EQ.'EMITSUM-REF='.AND.NCMT.LT.MAXCMT) THEN
		NCMT=NCMT+1
		CMNT(NCMT)=INLIN(13:)
		CMTTYP(NCMT)=1
	ELSEIF (INLIN(1:13).EQ.'EMITSUM-NOTE='.AND.NCMT.LT.MAXCMT) THEN
		NCMT=NCMT+1
		CMNT(NCMT)=INLIN(14:)
		CMTTYP(NCMT)=2
C
C	SUMMARY OUTPUT FILE IF NOT EMITSUM.CSV
C
	ELSEIF (INLIN(1:12).EQ.'EMITSUM-OUT=') THEN
		LOGFIL=INLIN(13:)
		LLOGFL=LNBLNK(LOGFIL)
	ENDIF
C
	GOTO 10
C
C	<EOF> ON INPUT UNIT. UNIT KEPT OPEN TO RE-READ TO GET
C	MECHANISM OVERRIDE ASSIGNMENTS AFTER DEFAULT ASSIGNMENTS READ
C	IN.
C
   12	CLOSE (UNIT=1)
	IF (MPIN) THEN
		IF (PRMFIL(1:6).EQ.'FILES=') THEN
			CALL PARSE0 (3,PRMFIL(7:),74)
			CALL PARSE1 (1,SUMIN,LSUMIN,60,PRMFIL(7:))
			CALL PARSE1 (2,EMITSM,LEMTSM,8,PRMFIL(7:))
			CALL PARSE1 (3,PRLIST,LPRLST,60,PRMFIL(7:))
		ELSE
			OPEN (UNIT=1,FILE=PRMFIL,STATUS='OLD',
     &			 ERR=9001)	! READONLY
			MPIN=.FALSE.
			GOTO 10
		ENDIF
	ENDIF
C
C
	ERRIND=.FALSE.
	IF (SUMIN(1:1).EQ.' ') THEN
		WRITE (6,*) 'No EMITSUM input file specfied'
		ERRIND=.TRUE.
	ENDIF
	IF (PRLIST(1:1).EQ.' ') THEN
		WRITE (6,*) 'No profile list specified.'
		ERRIND=.TRUE.
	ENDIF
	IF (EMITSM(1:1).EQ.' ') THEN
		WRITE (6,*) 'No emissions summary profile named'
		ERRIND=.TRUE.
	ENDIF
	IF (ERRIND) CALL EXIT (2)
C
C	OPEN PROFILE LIST FILE AND INPUT LIST OF PROFILES THAT MAY
C	BE USED.
C
	OPEN (UNIT=1,FILE=PRLIST(1:LPRLST),STATUS='OLD',ERR=902)
	NPRO=0
  20	READ (1,1,END=29) INLIN
	IF (INLIN(1:1).EQ.' '.OR.INLIN(1:1).EQ.'!') GOTO 20
	IF (NPRO.GT.MAXPRO) THEN
		WRITE (6,*) 'INCREASE MAXPRO IN EMITSUM.FOR'
		CALL EXIT (2)
	ENDIF
	CALL PARSE0 (2,INLIN,80)
	CALL PARSE1 (1,NAMEIN,I,8,INLIN)
	CALL PARSE1 (2,PNUM,J,4,INLIN)
	IF (PNUM(1:1).EQ.' ') THEN
		I1=MAX(1,I-3)
		PNUM=NAMEIN(I1:I)
	ENDIF
	NPRO=NPRO+1
	PRONAM(NPRO)=NAMEIN
	PRONUM(NPRO)=PNUM
	GOTO 20
   29	CLOSE (UNIT=1)
	IF (NPRO.EQ.0) THEN
		WRITE (6,*) 'No profiles listed in '//
     &		 PRLIST(1:LPRLST)
		CALL EXIT (2)
	ENDIF
C
C	READ COMPOUND LIST AND DESCRIPTIONS AND INITILIZE MASS TO 0
C
	IF (EDBUIC(1:1).EQ.' ') THEN
		INFILE='CMPDINFO.DAT'
		LINFIL=12
	ELSE
		INFILE=EDBUIC(1:LEDBUC)//'CMPDINFO.DAT'
		LINFIL=LEDBUC+12
	ENDIF
	OPEN (UNIT=1,FILE=INFILE(1:LINFIL),
     &	 STATUS='OLD',ERR=903)	! READONLY
  31	READ (1,1,END=903) INLIN
	IF (INLIN(1:1).NE.'.') GOTO 31
	NSPC=0
  34	READ (1,1,END=39) INLIN
	IF (NSPC.GT.MAXSPC) THEN
		WRITE (6,*) 'INCREASE MAXSPC IN EMITSUM.FOR'
		CALL EXIT (2)
	ENDIF
	IF (NSPC.EQ.MAXSPC) THEN
		WRITE (6,*) 'Increase MAXSPC in EMITSUM.FOR'
		CALL EXIT (2)
	ENDIF
	NSPC=NSPC+1
	READ (INLIN,35,END=39) SPCNAM(NSPC),SPCVOL(NSPC),SPCXMT(NSPC),
     &	 SPCDSC(NSPC)
  35	FORMAT (A16,12X,2X,A1,3X,A1,2X,A)
	SPCMAS(NSPC)=0.0
	GOTO 34
  39	CLOSE (UNIT=1)
C
C	OPEN SUMMARY PROFILE OUTPUT FILE AND WRITE HEADERS
C
	IF (ESMLOC(1:1).EQ.' ') THEN
		IF (EMCUIC(1:1).EQ.' ') THEN
			OUTFIL=EMITSM(1:LEMTSM)//'.EMC'
			LOUTFL=LEMTSM+4
		ELSE
			OUTFIL=EMCUIC(1:LEMCUC)//EMITSM(1:LEMTSM)
     &			 //'.EMC'
			LOUTFL=LEMCUC+LEMTSM+4
		ENDIF
	ELSE
		OUTFIL=ESMLOC(1:LESMLC)//EMITSM(1:LEMTSM)//'.EMC'
		LOUTFL=LESMLC+LEMTSM+4
	ENDIF
	OPEN (UNIT=2,FILE=OUTFIL(1:LOUTFL),STATUS='UNKNOWN')
	IF (DESC(1:1).NE.' ') WRITE (2,1) 'DESC='//DESC(1:LNBLNK(DESC))
	WRITE (2,1) 'EMITSUM='//EMITSM(1:LEMTSM)
	WRITE (2,1) 'EMITSUM-IN='//SUMIN(1:LSUMIN)
	WRITE (2,1) 'PROFILST='//PRLIST(1:LPRLST)
	IF (NCMT.GT.0) THEN
		DO 41 I=1,NCMT
		IF (CMTTYP(I).EQ.1) THEN
			WRITE (2,1) 'REF='//CMNT(I)(1:LNBLNK(CMNT(I)))
		ELSE
			WRITE (2,1) 'NOTE='//CMNT(I)(1:LNBLNK(CMNT(I)))
		ENDIF
   41		CONTINUE
	ENDIF
C
C	OPEN PROFILE TOTAL EMISSIONS FILE, GET AMOUNTS EMITTED, THEN
C	OPEN PROFILE .EMC FILE TO GET COMPOUNDS, THEN SUM COMPOUNDS.
C
	OPEN (UNIT=1,FILE=SUMIN(1:LSUMIN),STATUS='OLD',ERR=9011) ! READONLY
C
	TOTMAS=0.0
	TOTUNK=0.0
	NPROC=0
C
	OPEN (UNIT=4,FILE=LOGFIL,STATUS='UNKNOWN')
	WRITE (4,1) 'Profile,No.,Mass,Unknowns'
C
C	READ NEXT PROFLE AMOUNT RECORD AND SEE IF IT IS ONE TO BE
C	PROCESSED
C
 100	READ (1,1,END=199) INLIN
	CALL PARSE0 (3,INLIN,80)
	CALL PARSE1 (2,PTYP,I,4,INLIN)
	IF (PTYP.NE.PROTYP) GOTO 100
	CALL PARSE1 (1,PNUM,LPNUM,4,INLIN)
C
	DO 110 IPRO=1,NPRO
	IF (PRONUM(IPRO).EQ.PNUM) THEN
		NAMEIN=PRONAM(IPRO)
		LNAM=LNBLNK(NAMEIN)
		GOTO 111
	ENDIF
  110	CONTINUE
	GOTO 100
C
C	PROFILE FOUND FOR PROCESSING
C
  111	CALL PARSER (3,WTI,ERRIND,INLIN)
	IF (ERRIND) GOTO 901
	IF (WTI.EQ.0.0) GOTO 100
C
C	GET PROFILE COMPOSITION TO PROCESS
C
	IF (EMCUIC(1:1).EQ.' ') THEN
		EMCFIL=NAMEIN(1:LNAM)//'.EMC'
		LEMCFL=LNAM+4
	ELSE
		EMCFIL=EMCUIC(1:LEMCUC)//NAMEIN(1:LNAM)//'.EMC'
		LEMCFL=LEMCUC+LNAM+4
	ENDIF
	OPEN (UNIT=3,FILE=EMCFIL(1:LEMCFL),STATUS='OLD',
     &	 ERR=904)	! READONLY
  121	READ (3,1,END=904) INLIN
	IF (INLIN(1:1).NE.'.') GOTO 121
C
	PMAS=0.0
	PUNK=0.0
C
  130	READ (3,1,END=139) INLIN
	IF (INLIN(1:1).EQ.'!') GOTO 130
	CALL PARSE0 (2,INLIN,80)
	CALL PARSE1 (1,CNAME,I,16,INLIN)
	CALL PARSER (2,WTJ,ERRIND,INLIN)
	IF (ERRIND) GOTO 904
C
	PMAS=PMAS+WTJ
	IF ((CNAME(1:6).EQ.'INERT ').OR.(CNAME(1:7).EQ.'NONVOL ')
     &	 .OR.(CNAME(1:6).EQ.'NOASN ')) THEN
		PUNK=PUNK+WTJ
		GOTO 130
	ELSE
		DO 140 ISPC=1,NSPC
		IF (CNAME.EQ.SPCNAM(ISPC)) THEN
			SPCMAS(ISPC)=SPCMAS(ISPC)+(WTI*WTJ)
			GOTO 130
		ENDIF
  140		CONTINUE
		WRITE (6,*) 'Compound '//CNAME(1:LNBLNK(CNAME))
     &		 //' not found.'
		GOTO 904
	ENDIF
C
  139	CLOSE (UNIT=3)
	TOTMAS=TOTMAS+PMAS
	TOTUNK=TOTUNK+PUNK
	NPROC=NPROC+1
	WRITE (4,138) NAMEIN(1:LNAM),PNUM(1:LPNUM),PMAS,PUNK
 138	FORMAT (A,',',A,',',1PG11.5,',',G11.5)
	GOTO 100
C
C	DONE WITH INPUT
C
  199	CLOSE (UNIT=1)
	WRITE (6,*) 'Processed profiles sumarized in '
     &	 //LOGFIL(1:LLOGFL)
	CLOSE (UNIT=4)
C
	IF (TOTMAS.EQ.0.0) THEN
		WRITE (6,*) 'No mass processed.'
		CLOSE (UNIT=2,STATUS='DELETE')
		CALL EXIT (2)
	ENDIF	
C
C	IN PREPARATION FOR SORTING, REMOVE ZERO EMISSIONS SPECIES
C	ALSO NORMALIZE AND SUM UP NORMALIZED MASS (SHOULD BE 1)
C
	FAC=1.0/(TOTMAS-TOTUNK)
	TOTMS2=0.0
	TOTNV=0.0
	TOTXMT=0.0
	NSPC1=NSPC
	NSPC=0
	DO 210 ISPC=1,NSPC1
	WTI=SPCMAS(ISPC)
	IF (WTI.NE.0.0) THEN
		NSPC=NSPC+1
		SPCIND(NSPC)=ISPC
		WTI=WTI*FAC
		SPCMAS(ISPC)=WTI
		TOTMS2=TOTMS2+WTI
		IF (SPCVOL(ISPC).EQ.'0') TOTNV=TOTNV+WTI
		IF (SPCXMT(ISPC).EQ.'0') TOTXMT=TOTXMT+WTI
	ENDIF
  210	CONTINUE
C
C	SUMMARIZE AND OUTPUT RESULTS
C
	WRITE (2,2011) NPROC,TOTMAS,TOTUNK/TOTMAS,TOTMS2
 2011	FORMAT ('NOTE=PROFILES PROCESSED:',I4
     &	 /'NOTE=TOTAL MASS IN PROFILES:',G12.5
     &	 /'NOTE=MASS FRACTION UNKNOWNS:',F8.5
     &	 /'NOTE=NORMALIZED SO KNOWN MASS SUMS TO 1'
     &	 /'TOTWT=',F8.6)
	IF (TOTNV.NE.0.0) WRITE (2,2012) 'NONVOL',TOTNV
	IF (TOTXMT.NE.0.0) WRITE (2,2012) 'EXEMT',TOTXMT
 2012	FORMAT ('FRCT_',A,'=',F8.6)
	WRITE (2,1) '.'
	WRITE (2,1) '!UID_CODE       WT_FRACT     VOL VOC DESCRIPTION'
C
C	OUTPUT SORTED DATA
C
	DO 220 I=1,NSPC
	ISPC=SPCIND(I)
	WTI=SPCMAS(ISPC)
C
	IF (I.LT.NSPC) THEN
		DO 222 J=I+1,NSPC
		JSPC=SPCIND(J)
		IF (SPCMAS(JSPC).GT.WTI) THEN
			SPCIND(J)=ISPC
			ISPC=JSPC
			WTI=SPCMAS(JSPC)
		ENDIF
  222		CONTINUE
	ENDIF
  220	WRITE (2,221) SPCNAM(ISPC),WTI,SPCVOL(ISPC),SPCXMT(ISPC),
     &	 SPCDSC(ISPC)(1:LNBLNK(SPCDSC(ISPC)))
  221	FORMAT (A16,1PG12.6,2X,A1,3X,A1,2X,A)
C
C	DONE WITH OUTPUT	
C
	WRITE (6,*) 'File = '//OUTFIL(1:LOUTFL)//' created.'
	CALL EXIT (0)
C
C	ERRORS
C
 9001	WRITE (6,*) 'File = '//PRMFIL(1:LNBLNK(PRMFIL))
     &	 //' does not exist'
	GOTO 999
  901	WRITE (6,*) 'File = '//INFILE(1:LINFIL)
     &	 //' has errors or does not exist.'
	CLOSE (UNIT=2,STATUS='DELETE')
	GOTO 999
 9011	WRITE (6,*) 'File = '//SUMIN(1:LSUMIN)
     &	 //' has errors or does not exist.'
	CLOSE (UNIT=2,STATUS='DELETE')
	GOTO 999
  902	WRITE (6,*) 'File = '//PRLIST(1:LPRLST)//' does not exist.'
	GOTO 999
  903	WRITE (6,*) 'File = '//INFILE(1:LINFIL)//
     &	 ' has errors or does not exist.'
	GOTO 999
  904	WRITE (6,*) 'File = '//EMCFIL(1:LEMCFL)//
     &	 ' has errors or does not exist'
C
  999	CALL EXIT (2)
	END	
