	PROGRAM MECHPRO
C
C	MECHANISM PROCESSOR FOR LUMPED PARAMETER MECHANISMS
C
C	The Mechanism Processor (MechPro) program is used to prepare
C	mechanism files (giving reactions and rate constants or kinetic
C	parameters) when processing for adjustable SAPRC-99 or other
C	mechanisms where the parameters for some model species depend
C	on the distribution of compounds they represent, or to prepare
C	mechanisms where new model species have been added to represent
C	selected compounds.
C
C	Use: SPECPRO InputParmFile
C
C	where InputParmFile is the input file containing the
C	parameters controlling the program.
C
C	Calls RDASN to read compound and mechanism assignments
C	Calls RXNOUT to output the mechansim definiton data
C	Calls STSUBS subroutines
C
C	FILE NAMES AND GLOBAL PARAMETERS
C
	CHARACTER OUTFIL*60,RXNFMT*3
C
C	SPECIFICATIONS FOR MODEL SPECIES AND COMPOUNDS FOR
C	SPECPRO AND ITS SUBROUTINES
C	DMS AND PARAMETER SPECIFICATIONS
C
C	INCLUDE SPECSPEC.FOR	! INCLUDED IN DMSSPC
	INCLUDE 'DMSSPC.FOR'
C
C	TEMPERATURES FOR KP CONVERSIONS
C
	REAL TREF/300.0/,TREFS(3)/270.0,300.0,330.0/,RFAC/0.0019872/
C
C	OUTPUT FILE UNITS
C
	INTEGER LOGU	! LOG FILE
	INTEGER RXNU	! MECHANISM FILE
	INTEGER MSPU	! MODEL SPECIES LIST AND PARAMETERS
	PARAMETER (LOGU=4,RXNU=8,MSPU=9)
C
C	LOCAL SPECIFICATIONS
C
	REAL LMPAMW(MAXLMP),LMPANC(MAXLMP)
	CHARACTER INLIN*250,INFILE*80,CMPDI*16,NAMEIN*8,PRMOUT*60
	CHARACTER LMPNAM(MAXLMP)*8
	CHARACTER PCHR(3)*2/'*1','*2','*3'/
	LOGICAL ERRIND,EOF,OUTRXN,NOBASE,MPIN
C
C	SPECIES TABLE INFORMATION
C
	CHARACTER STABIN*60,STABOU*60
	CHARACTER EMISI*16,STINFO(MAXMS)*180,STINFI*180
	REAL MWI
	LOGICAL STDFD(MAXMS),USESTB
C
C	READ INPUT PARAMETERS
C
	CALL GETARG (1,INFILE)			! GET COMMAND LINE
	IF (INFILE(1:1).EQ.' ') THEN
		WRITE (6,*) 'Mechanism processor'
		WRITE (6,*) 'Use: MECHPRO inputfile'
		CALL EXIT (2)
	ENDIF
C
	EDBUIC=' '
	MECHID=' '
	OUTFIL=' '
	EMITSM=' '
	ESMLOC=' '
	MECLOC=' '
	EMCUIC=' '
	HVPRMF=' '
	PRDFIL=' '
	RXNFMT='MEC'
	NPRMF=0
	NRXNF=0
	NOBASE=.FALSE.
	STABIN=' '
	STABOU=' '
	USESTB=.FALSE.
	PRMOUT=' '
C
C	FIRST GET PARMS FROM MODELING.PRM, THEN FROM INPUT FILE
C
	MPIN=.TRUE.
	OPEN (UNIT=1,FILE='MODELING.PRM',STATUS='UNKNOWN')
C
   10	READ (1,1,END=12) INLIN
    1	FORMAT (A)
	IF (INLIN(1:5).EQ.'MECH=') THEN
		MECHID=INLIN(6:)
		LMECID=LNBLNK(MECHID)
	ELSEIF (INLIN(1:7).EQ.'RXNFMT=') THEN
		RXNFMT=INLIN(8:)
	ELSEIF (INLIN(1:12).EQ.'MECHPRO-OUT=') THEN
		OUTFIL=INLIN(13:)
		LOUTFL=LNBLNK(OUTFIL)
	ELSEIF (INLIN(1:7).EQ.'EMITDB=') THEN
		EDBUIC=INLIN(8:)
		LEDBUC=LNBLNK(EDBUIC)
	ELSEIF (INLIN(1:8).EQ.'EMITSUM=') THEN
		EMITSM=INLIN(9:)
		LEMTSM=LNBLNK(EMITSM)
	ELSEIF (INLIN(1:8).EQ.'MECHPRM=') THEN
		IF (NPRMF.EQ.MAXPMF) THEN
			WRITE (6,*) 'INCREASE MAXPMF IN MECHPRO.FOR'
			CALL EXIT (2)
		ENDIF
		NPRMF=NPRMF+1
		PRMFIL(NPRMF)=INLIN(9:)
	ELSEIF (INLIN(1:8).EQ.'MECHLOC=') THEN
		MECLOC=INLIN(9:)
		LMECLC=LNBLNK(MECLOC)
	ELSEIF (INLIN(1:8).EQ.'PHOTPRM=') THEN
		HVPRMF=INLIN(9:)
		LHVPMF=LNBLNK(HVPRMF)
	ELSEIF (INLIN(1:8).EQ.'PRODPRM=') THEN
		PRDFIL=INLIN(9:)
		LPRDFL=LNBLNK(PRDFIL)
	ELSEIF (INLIN(1:7).EQ.'RXNFMT=') THEN
		RXNFMT=INLIN(8:)
	ELSEIF (INLIN(1:7).EQ.'PRMOUT=') THEN
		PRMOUT=INLIN(8:)
C
C	LOCATION OF OUTPUT TOTAL EMISSIONS .EMC FILE IF NOT PROFEMC
C
	ELSEIF (INLIN(1:12).EQ.'EMITSUM-LOC=') THEN
		ESMLOC=INLIN(13:)
		LESMLC=LNBLNK(ESMLOC)
C
C	DEFAULT LOCATION FOR TOTAL EMISSIONS .EMC FILE IF EMITSUM-LOC
C	NOT SPECIFIED.
C
	ELSEIF (INLIN(1:8).EQ.'PROFEMC=') THEN
		EMCUIC=INLIN(9:)
		LEMCUC=LNBLNK(EMCUIC)
	ELSEIF (INLIN(1:6).EQ.'NOBASE') THEN
		NOBASE=.TRUE.
C
C	SPECIES TABLE NAMES (IF USED)
C
	ELSEIF(INLIN(1:14).EQ.'SPECIES_TABLE=') THEN
		CALL PARSE0 (2,INLIN(15:),80)
		CALL PARSE1 (1,STABIN,LSTBIN,60,INLIN(15:))
		CALL PARSE1 (2,STABOU,LSTBOU,60,INLIN(15:))
		IF (STABIN(1:1).EQ.' '.AND.STABOU(1:1).NE.' ') THEN
			WRITE (6,*) 'No species table input specified.'
     &			 //' Species table output ignored.'
			STABOU=' '
			USESTB=.FALSE.
		ELSEIF (STABIN(1:1).NE.' '.AND.STABOU(1:1).EQ.' ') THEN
			WRITE (6,*) 'No species table output specified'
     &			 //'. Species table input ignored.'
			STABIN=' '
			USESTB=.FALSE.
		ELSE
			USESTB=.TRUE.
		ENDIF
	ENDIF
	GOTO 10
C
C	EOF ON INPUT UNIT. UNIT KEPT OPEN TO RE-READ TO GET
C	MECHANISM OVERRIDE ASSIGNMENTS AFTER DEFAULT ASSIGNMENTS READ
C	IN.
C
C	<EOF> ON INPUT UNIT. UNIT KEPT OPEN TO RE-READ TO GET
C	MECHANISM OVERRIDE ASSIGNMENTS AFTER DEFAULT ASSIGNMENTS READ
C	IN.
C
   12	CONTINUE
	IF (MPIN) THEN
		CLOSE (UNIT=1)
		OPEN (UNIT=1,FILE=INFILE,STATUS='OLD',ERR=901)	! READONLY
		MPIN=.FALSE.
		GOTO 10
	ENDIF
C
	ERRIND=.FALSE.
	IF (MECHID(1:1).EQ.' ') THEN
		WRITE (6,*) 'MECH not specified'
		ERRIND=.TRUE.
	ENDIF
	IF (NPRMF.GT.0) THEN
C
C		DEFAULT FILE NAMES USED IF NOT SPECIFIED
C
		IF (PRDFIL(1:1).EQ.' ') THEN
			PRDFIL=MECHID(1:LMECID)//'PMS.DAT'
			LPRDFL=LMECID+7
		ENDIF
		IF (HVPRMF(1:1).EQ.' ') THEN
			HVPRMF=MECHID(1:LMECID)//'PHOT.DAT'
			LHVPMF=LMECID+8
		ENDIF
C
C		ERROR CHECKING
C
		IF (EMITSM(1:1).EQ.' ') THEN
			WRITE (6,*) 'Profile for lumping weighting '
     &			 //'not specified'
			ERRIND=.TRUE.
		ENDIF
		IF (RXNFMT.EQ.'MEC') THEN
			INDRXN=2
		ELSEIF (RXNFMT.EQ.'RXN') THEN
			INDRXN=1
		ELSE
			WRITE (6,*) RXNFMT,' is not a recognized '
     &			//' RXNFMT option. MEC used.'
			RXNFMT='MEC'
			INDRXN=2
		ENDIF
	ENDIF
C
	IF (ERRIND) THEN
		WRITE (6,*) 'MechPro run failed.'
		CALL EXIT (2)
	ENDIF
C
C	OPEN LOG FILE FOR STATUS MESSAGES
C
	IF (OUTFIL(1:1).EQ.' ') THEN
		OUTFIL='MECHPRO'
		LOUTFL=7
		WRITE (6,*) 'Output file not specified. MECHPRO '
     &		 //'used.'
	ENDIF
	OPEN (UNIT=LOGU,FILE=OUTFIL(1:LOUTFL)//'.LOG',STATUS='UNKNOWN')
	WRITE (LOGU,1) 'MechPro processing for '//MECHID(1:LMECID)
	WRITE (LOGU,1) 'Parameters:'
	IF (EDBUIC(1:1).NE.' ') WRITE(LOGU,1) ' EDBUIC='//
     &	 EDBUIC(1:LEDBUC)
	IF (ESMLOC(1:1).NE.' ') WRITE(LOGU,1) ' ESMLOC='//
     &	 ESMLOC(1:LESMLC)
	IF (MECLOC(1:1).NE.' ') WRITE(LOGU,1) ' MECLOC='//
     &	 MECLOC(1:LMECLC)
	IF (EMCUIC(1:1).NE.' ') WRITE(LOGU,1) ' EMCUIC='//
     &	 EMCUIC(1:LEMCUC)
	WRITE(LOGU,1) ' RXNFMT='//RXNFMT
C
C	READ LUMPING AND MECHANISM ASSIGNMENTS
C
C	READ COMPOUND AND MECHANISM ASSIGNMENTS USING RDASN
C	UNIT1 RE-READ TO GET MECHANISM OVER-RIDE ASSIGNMENTS.
C
	REWIND (UNIT=1)
	CALL RDASN (1,NPRMF.GT.0)
	CLOSE (UNIT=1)
C
C	CAN'T DO ANYTING WITH PARMS IF NO DMS
	IF (NDMS.EQ.0) NPRMF=0
C
	IF (NPRMF.GT.0) THEN
C
C		READ IN PHOTOLYSIS PARAMETERS.  IF NO "PHOTPRM" INPUT
C		IS GIVEN THEN ADJUSTABLE PHOTOLYSIS REACTIONS CANNOT
C		BE PROCESSED.
C
		NHV=0
		IF (MECLOC(1:1).EQ.' ') THEN
			INFILE=HVPRMF(1:LHVPMF)
			LINFIL=LHVPMF
		ELSE
			INFILE=MECLOC(1:LMECLC)//
     &			 HVPRMF(1:LHVPMF)
			LINFIL=LMECLC+LHVPMF
 		ENDIF
 		OPEN (UNIT=2,FILE=INFILE,STATUS='OLD',ERR=1802)	! READONLY
C		IF PHOT FILE EXISTS
 1801			READ (2,1,END=1809) INLIN
			IF (INLIN(1:1).EQ.'!'.OR.INLIN(1:1).EQ.' ')
     &			 GOTO 1801
			IF (NHV.GT.MAXHV) THEN
				WRITE (6,*) 'INCREASE MAXHV IN '
     &				 //'MECHPRO.FOR'
				CALL EXIT (2)
			ENDIF
			NHV=NHV+1
			HVPRM(NHV)=INLIN(1:2)
			HVPHF(NHV)=INLIN(4:)
			GOTO 1801
 1809			CLOSE (UNIT=2)
			IF (NHV.GT.0) WRITE (LOGU,1) 'Photolysis '
     &			 //'parameters read from '//INFILE(1:LINFIL)
C		ENDIF PHOT FILE EXISTS
 1802		CONTINUE
C
C		READ PRODUCT PARAMETER DEFINITIONS AND INFORMATION
C
		NPRD=0
 		IF (MECLOC(1:1).EQ.' ') THEN
 			INFILE=PRDFIL(1:LPRDFL)
			LINFIL=LPRDFL
 		ELSE
 			INFILE=MECLOC(1:LMECLC)//PRDFIL(1:LPRDFL)
			LINFIL=LMECLC+LPRDFL
 		ENDIF
 		OPEN (UNIT=2,FILE=INFILE,STATUS='OLD',ERR=904)	! READONLY
 1811		READ (2,1,END=1819) INLIN
		IF (INLIN(1:1).EQ.'!'.OR.INLIN(1:1).EQ.' ') GOTO 1811
		IF (NPRD.GT.MAXPRD) THEN
			WRITE (6,*) 'INCREASE MAXPRD IN '
     &			 //'DMSSPC.FOR'
			CALL EXIT (2)
		ENDIF
		NPRD=NPRD+1
		CALL PARSE0 (4,INLIN,80)
		CALL PARSE1 (1,PRDPRM(NPRD),I,2,INLIN)
		CALL PARSEI (2,PRDNC(NPRD),ERRIND,INLIN)
		IF (ERRIND) PRDNC(NPRD)=0
		CALL PARSE1 (INDRXN+2,PRDSPE(NPRD),I,8,INLIN)
		GOTO 1811
 1819		CLOSE (UNIT=2)
		IF (NPRD.GT.0) WRITE (LOGU,1) 'Product parameters read'
     &		 //' from '//INFILE(1:LINFIL)
C
C		INITIALIZE LUMPED GROUP COUNT.  THERE IS ONE LUMPED
C		GROUP FOR EACH MS WITH A NON-BLANK PARAMETER CODE.
C		LUMPED GROUPS WITH LMPDMS>0 USE PARMS OF A DMS,
C		THOSE WITH LMPDMS=0 USE WEIGHTED PARMS FOR COMPOUNDS
C		WITH THIS 
C
		NLMP=0
		DO 1601 IMS=1,NMS
			IF (MSLMP(IMS).EQ.-1) THEN
				IF (NLMP.EQ.MAXLMP) THEN
				    WRITE (6,*) 'INCREASE '
     &				     //'MAXLMP IN DMSSPC.FOR'
				    CALL EXIT (2)
				ENDIF
				NLMP=NLMP+1
				LMPNAM(NLMP)=MSNAM(IMS)
				LMPTOT(NLMP)=0.0
				LMPMS(NLMP)=IMS
				PRMDFD(NLMP)=.FALSE.
				LMPAMW(NLMP)=0.0	! AVG MW
				LMPANC(NLMP)=0.0	! AVG NC
				MSLMP(IMS)=NLMP
			ENDIF
 1601		CONTINUE
C
C		ELIMINATE SPECIES NOT REPRESENTED BY A POTENTIALLY
C		ADJUSTABLE MS AND DON'T HAVE A DMS ASSIGNED, SINCE
C		THEY'RE NOT NEEDED.  INITIALIZE WEIGHTING ARRAY
C		NOTE: AFTER THIS POINT, ONLY SPCNAM, SPCMS, SPCMSW,
C		SPCDMS, SPCMW, AND SPCWT ARE DEFINED, AND WE CAN  
C		SAFELY ASSUME THAT ALL COMPOUNDS HAVE BOTH LUMPED AND
C		DETAILED MS ASS'D.
C
		NSPC1=NSPC
		NSPC=0
		TOTMOL=0.0
C
		DO 164 I=1,NDMS
  164		DMSUSD(I)=.FALSE.
		DO 1641 I=1,NMS
		IF (MSDMS(I).GT.0) DMSUSD(MSDMS(I))=.TRUE.
 1641		CONTINUE
C
		DO 165 ISPC=1,NSPC1
		IMS=SPCMS(ISPC)
		IDMS=SPCDMS(ISPC)
		IF (SPCASN(ISPC).AND.SPCVOL(ISPC).AND.IMS.GT.0
     &		 .AND.IDMS.GT.0) THEN
			IF (MSLMP(IMS).GT.0) THEN
				NSPC=NSPC+1
				SPCNAM(NSPC)=SPCNAM(ISPC)
				SPCMW(NSPC)=SPCMW(ISPC)
				SPCNC(NSPC)=SPCNC(ISPC)
				SPCMS(NSPC)=IMS
				SPCMSW(NSPC)=SPCMSW(ISPC)
				SPCDMS(NSPC)=IDMS
				SPCMOL(NSPC)=0.0
				DMSUSD(IDMS)=.TRUE.
			ENDIF
		ENDIF
  165		CONTINUE
		IF (NSPC.EQ.0) THEN
			WRITE (6,*) 'No compounds have both lumped '
     &			 //'and detailed mechanism assignments.'
			CALL EXIT (2)
		ENDIF
C
C		READ WEIGHTING PROFILE. IGNORE RECORDS WITH
C		COMPOUNDS W/O DMS.
C
		IF (ESMLOC(1:1).EQ.' ') THEN
			IF (EMCUIC(1:1).EQ.' ') THEN
				INFILE=EMITSM(1:LEMTSM)//'.EMC'
				LINFIL=LEMTSM+4
			ELSE
				INFILE=EMCUIC(1:LEMCUC)//
     &				 EMITSM(1:LEMTSM)//'.EMC'
				LINFIL=LEMCUC+LEMTSM+4
			ENDIF
		ELSE
			INFILE=ESMLOC(1:LESMLC)//EMITSM(1:LEMTSM)
     &			 //'.EMC'
			LINFIL=LESMLC+LEMTSM+4
		ENDIF
		OPEN (UNIT=2,FILE=INFILE,STATUS='OLD',ERR=902)	! READONLY
 1701		READ (2,1,END=903) INLIN
		IF (INLIN(1:1).EQ.'.') GOTO 1701
C
  170		READ (2,1,END=179) INLIN
		IF (INLIN(1:1).EQ.' '.OR.INLIN(1:1).EQ.'!') GOTO 170
		CALL PARSE0 (2,INLIN,80)
		CALL PARSER (2,WTI,ERRIND,INLIN)
		IF (ERRIND.OR.WTI.EQ.0.0) GOTO 170
		CALL PARSE1 (1,CMPDI,I,16,INLIN)
		DO 110 ISPC=1,NSPC
		IF (CMPDI.EQ.SPCNAM(ISPC)) THEN
			WTI=WTI/SPCMW(ISPC)
			TOTMOL=TOTMOL+WTI
			SPCMOL(ISPC)=SPCMOL(ISPC)+WTI
			GOTO 170
		ENDIF
  110		CONTINUE
		GOTO 170
C
  179		CLOSE (UNIT=2)
		IF (TOTMOL.EQ.0.0) GOTO 903
		WRITE (LOGU,1) 'Parameter weighting profile read from '
     &		 //INFILE(1:LINFIL)
C
C		READ ALL PARAMETER FILES AND INPUT PARAMETER DATA
C
		NPRM=0
C
		WRITE (LOGU,'(''Parameter files read: '',I4)') NPRMF
		DO 200 IPRMF=1,NPRMF
		LPRMFL=LNBLNK(PRMFIL(IPRMF))
		EOF=.FALSE.
		IF (MECLOC(1:1).EQ.' ') THEN
			INFILE=PRMFIL(IPRMF)(1:LPRMFL)
		ELSE
			INFILE=MECLOC(1:LMECLC)//
     &			 PRMFIL(IPRMF)(1:LPRMFL)
		ENDIF
		OPEN (UNIT=2,FILE=INFILE,STATUS='OLD',ERR=904)	! READONLY
C		FIRST RECORD IS COMMENT
		READ (2,1,END=290) INLIN
C
  210		READ (2,1,END=290) INLIN
		IF (INLIN(1:1).EQ.'!'.OR.INLIN(1:1).EQ.' ') GOTO 210
		IF (INLIN(1:6).EQ.'PARMS ') THEN
			CALL PARSE0 (2,INLIN,80)
			CALL PARSEI (2,NL,ERRIND,INLIN)
			IF (ERRIND) GOTO 905
			NL=((NL-1)/8)+1
			DO 2101 I=1,NL
 2101			READ (2,*,END=290)
			GOTO 210
		ENDIF
C
C		NEW DMS FOR PARAMERERS
C
		NAMEIN=INLIN(1:8)
		DO 212 I=1,NDMS
		IF (NAMEIN.EQ.DMSNAM(I)) THEN
			IF (.NOT.DMSUSD(I)) GOTO 214
			IDMS=I
			IF (NPRM.EQ.0) GOTO 220
			DO 2121 IPRM=1,NPRM
 2121			ACTPRM(IPRM)=0.0
			ISHV=0
			GOTO 220
		ENDIF
  212		CONTINUE
C
C		DMS NOT FOUND OR USED
  214		READ (2,1,END=290) INLIN(1:1)
		IF (INLIN(1:1).EQ.' ') GOTO 210
		GOTO 214
C
C		INPUT PARMS FOR DMS # IDMS
C
  220		READ (2,1,END=240) INLIN
		IF (INLIN(1:1).EQ.'!') GOTO 220
		IF (INLIN(1:1).EQ.' ') GOTO 2401
C
		IF (INLIN(1:2).EQ.'K(') THEN
			CALL PARSE0 (4,INLIN,80)
			CALL PARSER (2,AFAC,ERRIND,INLIN)
			IF (ERRIND) GOTO 905
			CALL PARSR2 (3,EA,0.0,ERRIND,INLIN)
			IF (ERRIND) GOTO 905
			CALL PARSR2 (4,BFAC,0.0,ERRIND,INLIN)
			DO 2201 I=1,3
 2201			CALL ADDPRM (INLIN(3:4)//PCHR(I)//'  ',
     &			 AFAC*EXP(-EA/(RFAC*TREFS(I)))*((TREFS(I)/
     &			 TREF)**BFAC),0,IDUM)
		ELSE
			CALL PARSE0 (2,INLIN,80)
			CALL PARSE1 (1,NAMEIN,I,6,INLIN)
			CALL PARSER (2,PRMI,ERRIND,INLIN)
			IF (ERRIND) GOTO 905
			IF (NAMEIN(1:2).EQ.'HV') THEN
				IF (ISHV.GT.0) THEN
				ELSEIF (NHV.EQ.0) THEN
					GOTO 981
				ELSEIF (PRMI.NE.1.0) THEN
					GOTO 982
				ELSE
C
C					FIRST RECORD FOR PHOT PARMS
C					MUST GIVE PHOT TYPE AS DEFINED
C					IN PHOTPRM FILE. THAT ISN'T
C					A PARAMETER AS SUCH.
C
					DO 2202 I=1,NHV
					IF (NAMEIN(3:4).EQ.HVPRM(I))
     &					 THEN
						CALL ADDPRM (NAMEIN
     &						 (1:6),1.0,0,ISHV)
						GOTO 220
					ENDIF
 2202					CONTINUE
					GOTO 982
				ENDIF
			ELSE
				ISHV=0
			ENDIF

			CALL ADDPRM (NAMEIN(1:6),PRMI,ISHV,IDUM)
		ENDIF
		GOTO 220
C
C		END INPUT OF PARMS FOR THIS DMS
C
  240		EOF=.TRUE.
 2401		CONTINUE
C
C		SEE IF ANY LUMPED GROUP IS THIS DMS EXPLICLTIY
C
		DO 260 IMS=1,NMS
		IF (MSDMS(IMS).EQ.IDMS) THEN
			ILMP=MSLMP(IMS)
			PRMDFD(ILMP)=.TRUE.
			DO 261 IPRM=1,NPRM
  261			LMPPRM(ILMP,IPRM)=ACTPRM(IPRM)
			GOTO 269
		ENDIF
  260		CONTINUE
C
C		IF NO GROUP IS THIS EXPLICITLY,
C		GO THROUGH COMPOUNDS IN WEIGHTING MIXTURE
C		WITH THIS DMS AND ADD CONTRIBUTION
C
		DO 250 ISPC=1,NSPC
		IF (SPCDMS(ISPC).EQ.IDMS) THEN
			ILMP=MSLMP(SPCMS(ISPC))
			IF (.NOT.PRMDFD(ILMP)) THEN
			  WTI=SPCMOL(ISPC)*SPCMSW(ISPC)
			  IF (WTI.GT.0.0) THEN
			    LMPTOT(ILMP)=LMPTOT(ILMP)+WTI
			    LMPAMW(ILMP)=LMPAMW(ILMP)+(SPCMW(ISPC)*WTI)
			    LMPANC(ILMP)=LMPANC(ILMP)+
     &			     (FLOAT(SPCNC(ISPC))*WTI)
			    DO 251 IPRM=1,NPRM
			    IF (PRMPAR(IPRM).EQ.0) THEN
				LMPPRM(ILMP,IPRM)=LMPPRM(ILMP,IPRM)+
     &				 (WTI*ACTPRM(IPRM))
			    ELSE
				LMPPRM(ILMP,IPRM)=LMPPRM(ILMP,IPRM)+
     &				 (WTI*ACTPRM(PRMPAR(IPRM))*
     &				 ACTPRM(IPRM))
			    ENDIF
  251			    CONTINUE
			  ENDIF
			ENDIF
		ENDIF
  250		CONTINUE
C
C		DONE WITH THIS DMS
C
  269		IF (.NOT.EOF) GOTO 210
C
C		DONE WITH THIS PARAMETER FILE
C
  290		CLOSE (UNIT=2)
		WRITE (LOGU,1) ' '//INFILE(1:LNBLNK(INFILE))
  200		CONTINUE
C
C		DONE WITH ALL PARAMETER FILES.
C
C		CALCULATE PARAMETER AVERAGES FOR LUMPED GROUPS
C		PROCESSED USING WEIGHTING MIXTURE
C
		WRITE (LOGU,'(''Lumped groups: '',I4)') NLMP
C
		DO 310 ILMP=1,NLMP
		WTI=LMPTOT(ILMP)
		IF (PRMDFD(ILMP).OR.WTI.EQ.0.0) GOTO 310
		LMPAMW(ILMP)=LMPAMW(ILMP)/WTI
		LMPANC(ILMP)=LMPANC(ILMP)/WTI
C
		DO 320 IPRM=1,NPRM
		IF (PRMPAR(IPRM).NE.0) THEN
			WTIJ=LMPPRM(ILMP,PRMPAR(IPRM))
			IF (WTIJ.EQ.0.0) THEN
				LMPPRM(ILMP,IPRM)=0.0
			ELSE
				LMPPRM(ILMP,IPRM)=LMPPRM(ILMP,IPRM)/
     &				 (WTIJ*WTI)
			ENDIF
		ELSE
			LMPPRM(ILMP,IPRM)=LMPPRM(ILMP,IPRM)/WTI
		ENDIF
  320		CONTINUE
		PRMDFD(ILMP)=.TRUE.
  310		CONTINUE
	ENDIF
C
C	OPEN REACTION OUTPUT FILE AND COPY BASE MECHANISM, IF FOUND
C
	OPEN (UNIT=RXNU,FILE=OUTFIL(1:LOUTFL)//'.'//RXNFMT,
     &	 STATUS='UNKNOWN')
	OUTRXN=.FALSE.
C
	IF (NOBASE) GOTO 4103
	IF (MECLOC(1:1).EQ.' ') THEN
		INFILE=MECHID(1:LMECID)//'BASE.'//RXNFMT
		LINFIL=LMECID+8
	ELSE
		INFILE=MECLOC(1:LMECLC)//MECHID(1:LMECID)
     &		 //'BASE.'//RXNFMT
		LINFIL=LMECLC+LMECID+8
	ENDIF
	OPEN (UNIT=2,FILE=INFILE,STATUS='OLD',ERR=4103)	! READONLY
 4101		READ (2,1,END=4102) INLIN
		I1=LNBLNK(INLIN)
		IF (I1.EQ.0) THEN
			WRITE (RXNU,1) '!'
			GOTO 4101
		ENDIF
		WRITE (RXNU,1) INLIN(1:I1)
		GOTO 4101
 4102		CLOSE (UNIT=2)
		WRITE (LOGU,1) 'Base mechanism copied from '
     &		 //INFILE(1:LINFIL)
		OUTRXN=.TRUE.
	GOTO 4104
 4103		WRITE (LOGU,1) 'No base mechanism file found.'
 4104	CONTINUE
C
C	OPEN MODEL SPECIES FILE AND OUTPUT HEADER
C
	OPEN (UNIT=MSPU,FILE=OUTFIL(1:LOUTFL)//'.MSP',STATUS='UNKNOWN')
	WRITE (MSPU,1) '!NAME      nC       MWT  DMS'
C
C	OUTPUT INFORMATION ON MODEL SPECIES.  ALSO,
C	OUTPUT REACTIONS FOR ADJUSTABLE MODEL SPECIES
C	(USE DEFAULTS IN (mech)(modspe).(RXNFMT) IF PARMS NOT DEFINED)
C
	DO 400 IMS=1,NMS
	IF (MSDMS(IMS).GT.0) THEN
		WRITE (MSPU,'(A8,I5,F10.2,2X,A8)') 
     &		 MSNAM(IMS),MSNC(IMS),MSMWT(IMS),DMSNAM(MSDMS(IMS))
	ELSE
		WRITE (MSPU,'(A8,I5,F10.2)') 
     &		 MSNAM(IMS),MSNC(IMS),MSMWT(IMS)
	ENDIF
C
	ILMP=MSLMP(IMS)
	IF (ILMP.GT.0) THEN
		IF (PRMDFD(ILMP)) THEN
C			THIS MS HAS PARAMETERS DEFINED
			IF (MSDMS(IMS).GT.0) THEN
				WRITE (RXNU,1) '!'
				WRITE (RXNU,1) '! ******************'
				WRITE (RXNU,1) '! Reactions for '
     &				 //MSNAM(IMS)//' created from para'
     &				 //'meters for '//DMSNAM(MSDMS(IMS))
			ELSE
				WRITE (RXNU,1) '!'
				WRITE (RXNU,1) '! ******************'
				WRITE (RXNU,1) '! Reactions for '
     &				 //MSNAM(IMS)//' created from para'//
     &				 'meters for compounds in the '//
     &				 EMITSM(1:LEMTSM)//' mixture.'
			ENDIF
			OUTRXN=.TRUE.
			CALL RXNOUT(RXNU,INDRXN,IMS,ILMP)
			WRITE (RXNU,1) '! ******************'
			WRITE (LOGU,1) ' '//RXNFMT//' for '
     &			 //MSNAM(IMS)//' created from parameters.'
			OUTRXN=.TRUE.
			GOTO 400
		ENDIF
	ENDIF
	IF (ILMP.NE.0) THEN
C		THIS MS HAS ADJUSTABLE PARAMS BUT NO PARMS
C		GET DEFAULT MECHANISM FROM FILE
		NAMEIN=MSNAM(IMS)
		LMSNAM=LNBLNK(NAMEIN)
		IF (MECLOC(1:1).EQ.' ') THEN
			INFILE=MECHID(1:LMECID)//NAMEIN(1:LMSNAM)
     &			 //'.'//RXNFMT
			LINFIL=LMECID+LMSNAM+4
		ELSE
			INFILE=MECLOC(1:LMECLC)//MECHID(1:LMECID)
     &			 //NAMEIN(1:LMSNAM)//'.'//RXNFMT
			LINFIL=LMECLC+LMECID+LMSNAM+4
		ENDIF
		OPEN (UNIT=2,FILE=INFILE,STATUS='OLD',ERR=403)	! READONLY
		WRITE (RXNU,1) '!'
		WRITE (RXNU,1) '! ******************'
		WRITE (RXNU,1) '! Reactions for '//MSNAM(IMS)
     &		 //' copied from '//INFILE(1:LINFIL)
  402		READ (2,1,END=409) INLIN
		I1=LNBLNK(INLIN)
		IF (I1.EQ.0) THEN
			WRITE (RXNU,1) '!'
			GOTO 402
		ENDIF
		WRITE (RXNU,1) INLIN(1:I1)
		GOTO 402
  409		CLOSE (UNIT=2)
		WRITE (RXNU,1) '! ******************'
		WRITE (LOGU,1) ' '//RXNFMT//' for '//MSNAM(IMS)
     &		 //' copied from '//INFILE(1:LINFIL)
		OUTRXN=.TRUE.
		GOTO 400
  403		WRITE (RXNU,1) '!'
		WRITE (RXNU,1) '! ******************'
		WRITE (RXNU,1) 'ERROR: No reactions given for '
     &		 //NAMEIN(1:LMSNAM)
		WRITE (RXNU,1) '! ******************'
		WRITE (LOGU,1) ' '//RXNFMT//' for '//MSNAM(IMS)
     &		 //' **ERROR** NO '//RXNFMT//' FILE FOUND'
	ENDIF
  400	CONTINUE
C
C	OUTPUT AVG MWT AND NC'S CALCULATED
C
	WRITE (LOGU,*) 'MSname  AvgMwt    AvgnC'
	DO 420 IMS=1,NMS
	ILMP=MSLMP(IMS)
	IF (ILMP.GT.0) THEN
		IF (LMPAMW(ILMP).GT.0.0) WRITE (LOGU,421) 
     &		 MSNAM(IMS),LMPAMW(ILMP),LMPANC(ILMP)
	ENDIF
  421	FORMAT (1X,A8,F6.2,F9.3)
  420	CONTINUE
C
C	MECHANISTIC PARAMETERS (IF ANY) HAVE BEEN PROCESSED AT THIS
C	POINT
C
	IF (PRMOUT(1:1).NE.' '.AND.NPRM.GT.0.AND.NLMP.GT.0) THEN
		OPEN (UNIT=9,FILE=PRMOUT,STATUS='UNKNOWN')
		WRITE (9,'(6X,40A11)') (LMPNAM(I),I=1,NLMP)
		DO 431 I=1,NPRM
  431		WRITE (9,'(A6,1X,1P40E11.3)') PARMID(I),
     &		 (LMPPRM(J,I),J=1,NLMP)
		CLOSE (UNIT=9)
		WRITE (6,*) 'File = '//PRMOUT(1:LNBLNK(PRMOUT))//' created.'
	ENDIF
C
	IF (OUTRXN) THEN
C
C		WRITE CLOSING INFO, IF FILE EXISTS, THEN CLOSE FILE
C
		IF (NOBASE) THEN
			CLOSE (UNIT=RXNU)
		ELSE
			CALL RXNEND(RXNU,INDRXN)
		ENDIF
C
		WRITE (LOGU,1) RXNFMT//' output in file = '//
     &		 OUTFIL(1:LOUTFL)//'.'//RXNFMT
		WRITE (6,*) 'File = '//OUTFIL(1:LOUTFL)//'.'//RXNFMT
     &		 //' created.'
	ELSE
C
C		IF NO RXN OUTPUT, THEN DELETE RXN FILE
		CLOSE (UNIT=RXNU,STATUS='DELETE')
		WRITE (LOGU,1) 'No '//RXNFMT//' file created.'
	ENDIF
C
	CLOSE (UNIT=MSPU)
	WRITE (LOGU,1) 'Model species listed in '//OUTFIL(1:LOUTFL)//
     &	 '.MSP'
	WRITE (6,*) 'File = '//OUTFIL(1:LOUTFL)//'.MSP created.'
C
C	INPUT, UPDATE, AND OUTPUT SPECIES TABLE, IF APPLICABLE.
C
	IF (USESTB) THEN
		OPEN (UNIT=MSPU,FILE=STABIN(1:LSTBIN),STATUS='OLD',
     &		 ERR=508)
		READ (MSPU,1,END=509) INLIN
		OPEN (UNIT=RXNU,FILE=STABOU(1:LSTBOU),STATUS='UNKNOWN')
		WRITE (RXNU,1) INLIN(1:LNBLNK(INLIN))
C
		DO 501 I=1,NSP
  501		STDFD(I)=.FALSE.
		GOTO 510
C
  508		WRITE (6,*) 'Error opening species table input.'
		WRITE (6,*) 'File=',STABIN(1:LSTBIN),' does not exist.'
		GOTO 599
  509		WRITE (6,*) 'Species table input file is empty.'
		CLOSE (UNIT=MSPU)
		GOTO 599
C
  510		READ (MSPU,1,END=519) INLIN
		CALL PARSE0 (4,INLIN,250)
		CALL PARSE1 (1,NAMEIN,I1,8,INLIN)
		CALL PARSER (2,MWI,ERRIND,INLIN)
		IF (ERRIND) GOTO 581
		CALL PARSE1 (3,EMISI,LEMISI,16,INLIN)
		IF (EMISI(1:1).EQ.' ') THEN
			EMISI=','
			LEMISI=1
		ELSE
			LEMISI=LEMISI+1
			EMISI(LEMISI:LEMISI)=','
		ENDIF
		CALL PARSE2 (4,STINFI,LSTINF,180,INLIN)
C
		DO 520 I=1,NMS
		IF (NAMEIN.EQ.MSNAM(I)) THEN
			STDFD(I)=.TRUE.
			STINFO(I)=STINFI
			IF (MWI.LT.0.0) THEN
				ILMP=MSLMP(I)
				IF (ILMP.GT.0) THEN
					IF (LMPAMW(ILMP).GT.0.0) 
     &					 MSMWT(I)=LMPAMW(ILMP)
				ENDIF
			ENDIF
			MWI=MSMWT(I)
			GOTO 522
		ENDIF
  520		CONTINUE
		IF (NMS.EQ.MAXMS) THEN
			WRITE (6,*) 'INCREASE MAXMS'
			GOTO 582
		ENDIF
		NMS=NMS+1
		MSNAM(NMS)=NAMEIN
		STINFO(NMS)=STINFI
		STDFD(NMS)=.TRUE.
		MWI=ABS(MWI)
C
  522		WRITE (RXNU,523) NAMEIN(1:I1),MWI,EMISI(1:LEMISI),
     &		 STINFI(1:LSTINF)
  523		FORMAT (A,',',F6.2,',',2A)
		GOTO 510
C
  519		CLOSE (UNIT=MSPU)
C
		DO 530 IMS=1,NMS
		IF (STDFD(IMS)) GOTO 530
		NAMEIN=MSLIK(IMS)
		STINFI='?'
		IF (NAMEIN.NE.' ') THEN
			DO 532 I=1,NMS
			IF (NAMEIN.EQ.MSNAM(I)) THEN
				STINFI=STINFO(I)
				GOTO 533
			ENDIF
  532			CONTINUE
		ENDIF
  533		NAMEIN=MSNAM(IMS)
		I1=LNBLNK(NAMEIN)
		WRITE (RXNU,523) NAMEIN(1:I1),MSMWT(IMS),
     &		 NAMEIN(1:I1)//',',STINFI(1:LNBLNK(STINFI))
  530		CONTINUE
C
		CLOSE (UNIT=RXNU)
		WRITE (6,*) 'New Species Table = '//STABOU(1:LSTBOU)
		WRITE (LOGU,1) 'Species table = '//STABOU(1:LSTBOU)
	ENDIF
	GOTO 599
C
  581	WRITE (6,*) 'Error processing species table input. Bad line:'
	WRITE (6,*) INLIN(1:LNBLNK(INLIN))
  582	WRITE (6,*) 'Species table processing aborted'
	CLOSE (UNIT=RXNU,STATUS='DELETE')
	CLOSE (UNIT=MSPU)
C
  599	CONTINUE
C
C	DONE
C
	CLOSE (UNIT=LOGU)
	WRITE (6,*) 'File = '//OUTFIL(1:LOUTFL)//'.LOG created.'
	CALL EXIT (0)
C
C	ERRORS
C
  901	WRITE (6,*) 'Error opening input file: '//
     &	 INFILE(1:LNBLNK(INFILE))
	GOTO 999
  902	WRITE (6,*) 'Lumped weighting file = '//
     &	 INFILE(1:LNBLNK(INFILE))//' does not exist.'
	GOTO 999
  903	WRITE (6,*) 'Lumped weighting file = '//EMITSM(1:LEMTSM)
     &	 //'.EMC has errors or no data.'
	GOTO 999
  904	WRITE (6,*) 'Error opening mechanistic parameter file = '
     &	 //INFILE(1:LNBLNK(INFILE))
	GOTO 999
  905	WRITE (6,*) 'Error processing mechanistic parameter file = '
     &	 //INFILE(1:LNBLNK(INFILE))
	WRITE (6,*) 'Bad record: '//INLIN(1:LNBLNK(INLIN))
	GOTO 999
C
  981	WRITE (6,*) 'Photolysis parameters not defined.'
  	GOTO 989
  982	WRITE (6,*) 'First HV parameter does not give recoginzed'
     &	 //' photolysis type with value of 1.0'
  989	WRITE (6,*) 'Cannot process photolysis parameter for '//
     &	 DMSNAM(IDMS)//' in '//INFILE(1:LNBLNK(INFILE))
	WRITE (6,*) 'Record = '//INLIN(1:LNBLNK(INLIN))
C
  999	CALL EXIT (2)
	END
C
C	----------------------------------------------
	SUBROUTINE ADDPRM (CHPRM,PRMVAL,IPAR,INDPRM)
C
C	ADDS THE SPECIFIED PARAMETERE TO THE LIST OF PARAMETERS
C	THAT ARE CURRENTLY BEING PROCESSED, WITH THE VALUE PUT IN
C	THE "ACTPRM" ARRAY.  ADDS NEW PARAMETERS TO LIST IF NEEDED.
C
	CHARACTER CHPRM*6	! PARAMETER CODE (INPUT)
	REAL PRMVAL		! PARAMETER VALUE (INPUT)
	INTEGER IPAR		! DEFAULT PARENT PARAMETER (INPUT)
	INTEGER INDPRM		! PARAMETER NO (OUTPUT)
C
	INCLUDE 'DMSSPC.FOR'
C
	CHARACTER CHKPRM*2
C
	IF (NPRM.GT.0) THEN
		DO 10 I=1,NPRM
		IF (CHPRM.EQ.PARMID(I)) THEN
			ACTPRM(I)=PRMVAL
			INDPRM=I
			RETURN
		ENDIF
   10		CONTINUE
	ENDIF
	IF (NPRM.EQ.MAXPRM) THEN
		WRITE (6,*) 'INCREASE MAXPRM IN DMSSPC.FOR'
		CALL EXIT (2)
	ENDIF
	NPRM=NPRM+1
	INDPRM=NPRM
	PARMID(NPRM)=CHPRM
	ACTPRM(NPRM)=PRMVAL
	IF (CHPRM(5:6).EQ.'  '.OR.NPRM.EQ.1) THEN
		PRMPAR(NPRM)=IPAR
	ELSE
		DO 15 I=1,NPRM-1
		IF (PARMID(I)(1:4).EQ.CHPRM(1:4).AND.PARMID(I)(5:6).
     &		 EQ.'  ') THEN
			PRMPAR(NPRM)=I
			GOTO 16
		ENDIF
   15		CONTINUE
	ENDIF
C
C	INITILAIZE THIS PARM VALUE FOR THE LUMPED GROUPS
C
   16	DO 20 I=1,NLMP
   20	LMPPRM(I,NPRM)=0.0
C
C	DETERMINE PRODUCTS ASSOCIATED WITH EACH PARAMETER
C
	CHKPRM=CHPRM(5:6)
	IF (CHKPRM.EQ.'  ') CHKPRM=CHPRM(3:4)
	DO 331 I=1,NPRD
	IF (CHKPRM.EQ.PRDPRM(I)) THEN
		PRMPRD(NPRM)=I
		RETURN
	ENDIF
  331	CONTINUE
	PRMPRD(NPRM)=0
 	RETURN
	END
