	PROGRAM PROFPROT
C
C	PROFILE PRE-PROCESSOR -- for Texas point sources with combined "contam" and
C	SAROAD codes.
C	Reads CSV file in format:
C	"tceq_contam,species,saroad,mod_epa_prof,source,profid,wt_fraction"
C
C	SarType's fixed at "TXC" for "tceq_contam" and "TX" for "Saroad"
C
C	Use: PROFPROT InputFile OutputFile
C	
C	Where InputFile is the name of the input file with records of 
C	the type ProfileID SAROAD masss fraction
C	and OutputFile is the output file which will have similar
C	format except it will give mass fractions of compounds in the 
C	speciation database.
C	Outputs diagnostic information in ProfProT.csv
C
C	Other parameters are read from modeling.prm
C
C	-----------------------------------------------------------
C	Calls STSUBS subroutines
C
C	SAROAD ASSIGNMENT DATA
C
	PARAMETER (MAXSAR=2000)
	CHARACTER SARNAM(MAXSAR,2)*12,SARID(MAXSAR,2)*16
	INTEGER NSARID(2)
	CHARACTER SARTYP(2)*3/'TXC','TX '/
C
C	ASSIGNMENT OF SPECIES TO UNIQUE ID'S (UID'S)
C
	PARAMETER (MAXSPC=8000,MAXUID=3000,MAXCMP=2000)
	CHARACTER UIDLST(MAXUID)*16,UIDSPC(MAXSPC)*16,CMPID(MAXCMP)*16
	REAL SPCWT(MAXSPC)
	INTEGER NUID1(MAXUID),NUID2(MAXUID)
	INTEGER NUID,NSPCID
C
C	COMPOUND DESCRIPTION INFORMATION
C
C	EMISSIONS FILE DATA
C
	INTEGER NCMP
	REAL TOTWT,CMPWT(MAXCMP)
C
C	LOCAL VARIABLES
C
	CHARACTER SARI1*16,SARI2*16,UID*16,OLDUID*16,CMPI*16
	CHARACTER PROFID*16,PROFIN*16
	LOGICAL EOF
	CHARACTER EDBUIC*40
	CHARACTER INLIN*200,INFILE*60
	CHARACTER OUTFIL*60
	LOGICAL ERRIND
C
C	OPEN INPUT FILE AND READ PARAMETERS
C
C	CALL GETCL (INLIN)			! GET COMMAND LINE
C	CALL PARSE0 (3,INLIN,80)
C	CALL PARSE1 (1,INFILE,LINFIL,60,INLIN)
C	CALL PARSE1 (2,OUTFIL,LOUTFL,60,INLIN)
	CALL GETARG (1,INFILE)
	CALL GETARG (2,OUTFIL)
	IF (INFILE(1:1).EQ.' '.OR.OUTFIL(1:1).EQ.' '.OR.
     &	 INFILE(1:1).EQ.'?') THEN
		WRITE (6,*) 'Converts Texas point source profiles'
     &		 //' from Contam and Saroad to SDB compounds'
		WRITE (6,*) 'Use: ProfProT InputFile OutputFile'
		CALL EXIT (2)
	ENDIF
	LINFIL=LNBLNK(INFILE)
	LOUTFL=LNBLNK(OUTFIL)
C
	EDBUIC=" "
C
C	GET OPTIONS FROM MODELING.PRM
C
	OPEN (UNIT=1,FILE='MODELING.PRM',STATUS='UNKNOWN')
C
   10	READ (1,1,END=12) INLIN
    1	FORMAT (A)
C
C	DEFAULT NATIVE CATEGORIZATION TYPE (CAN BE OVER-WRITTEN
C	BY INPUT ON THE COMMAND LINE
C
C	LOCATION OF THE SPECASN.DAT AND SAROID ASSIGNMEN FILES
C
	IF (INLIN(1:7).EQ.'EMITDB=') THEN
		EDBUIC=INLIN(8:)
		LEDBUC=LNBLNK(EDBUIC)
C
	ENDIF
C
C	END OF PARAMETER INPUT
C
	GOTO 10
C
   12	CLOSE (UNIT=1)
C
C
C	READ ASSIGNMENT DATA USED
C
C	READ ID-TO=COMPOUND ASSIGNMENTS
C
	IF (EDBUIC(1:1).EQ.' ') THEN
		INLIN='SPECASN.DAT'
	ELSE
		INLIN=EDBUIC(1:LEDBUC)//'SPECASN.DAT'
	ENDIF
	OPEN (UNIT=2,FILE=INLIN,STATUS='OLD',ERR=902)
	READ (2,1,END=903) INLIN
  21	READ (2,1,END=903) INLIN
	IF (INLIN(1:1).NE.'.') GOTO 21
	NSPCID=0
	NUID=0
	OLDUID=" "
  22	READ (2,1,END=28) INLIN
	IF (INLIN(1:1).EQ.' '.OR.INLIN(1:1).EQ.'!') GOTO 22
	IF (NSPCID.EQ.MAXSPC) THEN
		WRITE (6,*) 'TOO MANY UID-TO-COMPOUND ASSIGNMENTS'
		WRITE (6,*) 'INCREASE MAXSPC IN PROFPROT.FOR'
		GOTO 904
	ENDIF
	NSPCID=NSPCID+1
	READ (INLIN,23,ERR=904) UID,UIDSPC(NSPCID),
     &	 SPCWT(NSPCID)
   23	FORMAT(2A16,F10.8)
	IF (UID.NE.OLDUID) THEN
		IF (NUID.GT.0) NUID2(NUID)=NSPCID-1
		IF (NUID.EQ.MAXUID) THEN
			WRITE (6,*) 'TOO MANY UID''S.'
			WRITE (6,*) 'INCREASE MAXUID IN PROFPROT.FOR'
			GOTO 904
		ENDIF
		NUID=NUID+1
		UIDLST(NUID)=UID
		NUID1(NUID)=NSPCID
		OLDUID=UID
	ENDIF
	GOTO 22
   28	IF (NUID.GT.0) NUID2(NUID)=NSPCID-1
	CLOSE (UNIT=2)
C
C	READ SAROAD ASSIGNMENTS
C
	DO 1001 ISAR=1,2
		LSARTP=LNBLNK(SARTYP(ISAR))
		IF (EDBUIC(1:1).EQ.' ') THEN
			INLIN=SARTYP(ISAR)(1:LSARTP)//'SARID.DAT'
		ELSE
			INLIN=EDBUIC(1:LEDBUC)//
     &			 SARTYP(ISAR)(1:LSARTP)//'SARID.DAT'
		ENDIF
		OPEN (UNIT=2,FILE=INLIN,STATUS='OLD',ERR=905)
		READ (2,1,END=905) INLIN
 121		READ (2,1,END=906) INLIN
		IF (INLIN(1:1).NE.'.') GOTO 121
		NSARID(ISAR)=0
 122		READ (2,1,END=128) INLIN
		IF (INLIN(1:1).EQ.' '.OR.INLIN(1:1).EQ.'!') 
     &		 GOTO 122
		IF (NSARID(ISAR).EQ.MAXSAR) THEN
  			WRITE (6,*) 'TOO MANY SAROAD '
     &			 //'ASSIGNMENTS'
  			WRITE (6,*) 'INCREASE MAXSAR'
			GOTO 906
		ENDIF
		CALL PARSE0 (2,INLIN,80)
		NSARID(ISAR)=NSARID(ISAR)+1
		CALL PARSE1 (1,SARNAM(NSARID(ISAR),ISAR),I,16,INLIN)
		CALL PARSE1 (2,SARID(NSARID(ISAR),ISAR),I,16,INLIN)
		IF (SARID(NSARID(ISAR),ISAR)(1:1).EQ.' ') 
     &		 NSARID(ISAR)=NSARID(ISAR)-1
		GOTO 122
  128		CLOSE (UNIT=2)
 1001	CONTINUE
C
C	OPEN INPUT AND OUTPUT FILES
C
	OPEN (UNIT=1,FILE=INFILE,STATUS='OLD',ERR=111)
	GOTO 112
  111	WRITE (6,*) 'File = '//INFILE(1:LINFIL)//
     &	 ' does not exist'
	CALL EXIT (2)
C
  112	OPEN (UNIT=2,FILE=OUTFIL,STATUS='UNKNOWN')
	WRITE (2,1) 'Input='//INFILE(1:LINFIL)
	OPEN (UNIT=3,FILE='PROFPROE.CSV',STATUS='UNKNOWN')
	WRITE (3,*) 'Profile,Output Mass,Input Mass,'
     &	 //'Unassigned,Nonvolatile'
C
	PROFIN=' '
	EOF=.FALSE.
	NFILES=0
C
   99	READ (1,1,END=180) INLIN
	CALL PARSE0 (7,INLIN,200)
	CALL PARSE1 (6,PROFIN,I,16,INLIN)
	IF (PROFIN(1:1).EQ.' ') GOTO 99
C
C	NEW PROFILE -- FROM LAST LINE READ
C
  100	NCMP=0
	TOTWT=0.0
	PROFID=PROFIN
	GOTO 131
C
C	READ NEXT LINE OF EMI FILE AND PROCESS
C
  130	READ (1,1,END=180) INLIN
	IF (INLIN(1:1).EQ.'!') GOTO 130
	CALL PARSE0 (7,INLIN,200)
	CALL PARSE1 (6,PROFIN,I,16,INLIN)
	IF (PROFIN.EQ.' ') GOTO 130
	IF (PROFIN.NE.PROFID) GOTO 181
C
  131	CALL PARSE1 (1,SARI1,I,16,INLIN)
	CALL PARSE1 (3,SARI2,I,16,INLIN)
	CALL PARSER (7,WTI,ERRIND,INLIN)
	IF (ERRIND) GOTO 130
	TOTWT=TOTWT+WTI
C
C	GET UNIQUE ID.  CHECK CONTAM FIRST, THEN SAROAD
C
	IF (SARI1(1:1).NE.' ') THEN
		DO 132 I=1,NSARID(1)
		IF (SARNAM(I,1).EQ.SARI1) THEN
			UID=SARID(I,1)
			GOTO 134
		ENDIF
  132		CONTINUE
		WRITE (6,*) 'No assignment for CONTAM ',SARI1
		GOTO 999
	ELSE
C
		DO 1321 I=1,NSARID(2)
		IF (SARNAM(I,2).EQ.SARI2) THEN
			UID=SARID(I,2)
			GOTO 134
		ENDIF
 1321		CONTINUE
		WRITE (6,*) 'No assignment for SAROAD ',SARI2
		GOTO 999
	ENDIF
C
  134	CONTINUE
C
C	GET COMPOUND ASSIGNMENTS FOR UNIQUE ID UID.
C
	DO 140 I=1,NUID
	IF (UID.EQ.UIDLST(I)) THEN
		DO 142 J=NUID1(I),NUID2(I)
			CMPI=UIDSPC(J)
			WTJ=WTI*SPCWT(J)
			IF (NCMP.GT.0) THEN
				DO 144 K=1,NCMP
				IF (CMPI.EQ.CMPID(K)) THEN
					CMPWT(K)=CMPWT(K)+WTJ
 					GOTO 142
 				ENDIF
  144				CONTINUE
			ENDIF
			IF (NCMP.EQ.MAXCMP) THEN
				WRITE (6,*) 
     &				 'TOO MANY COMPOUNDS.'
				WRITE (6,*)
     &				 'INCREASE MAXCMP IN '
     &				 //'EMITOEMC.FOR'
				GOTO 906
			ENDIF
			NCMP=NCMP+1
			CMPID(NCMP)=CMPI
			CMPWT(NCMP)=WTJ
  142		CONTINUE
		GOTO 130
	ENDIF
  140	CONTINUE
	WRITE (6,*) 'No compound assignment for UID ',UID
	GOTO 999
C
C	NORMAL END OF EMI FILE INPUT.
C
  180	EOF=.TRUE.
  181	IF (TOTWT.EQ.0.0) GOTO 190
	TOTWT2=0.0
	UNAWT=0.0
	UNVWT=0.0
	DO 182 I=1,NCMP
	WTI=CMPWT(I)
	IF (CMPID(I)(1:6).EQ.'NOASN ') THEN
		UNAWT=UNAWT+WTI
	ELSEIF (CMPID(I)(1:7).EQ.'NONVOL ') THEN
		UNVWT=UNVWT+WTI
	ENDIF
  182	TOTWT2=TOTWT2+WTI
C
	WRITE (2,1) '..'
	WRITE (2,1) PROFID(1:LNBLNK(PROFID))
	DO 185 I=1,NCMP
	IF (CMPWT(I).NE.0.0) WRITE (2,186) CMPID(I),CMPWT(I)
  185	CONTINUE
  186	FORMAT (A16,1PE10.4)
	WRITE (3,188) PROFID,TOTWT2,TOTWT,UNAWT,UNVWT
  188	FORMAT (A16,6(',',1PE10.4))
	NFILES=NFILES+1
C
  190	IF (EOF) GOTO 800
	GOTO 100
C
C	ERROR PROCESSING DATA FOR THE INPUT .EMI FILE
C
C
C	END OF INPUT.
C
  800	CLOSE (UNIT=1)
	CLOSE (UNIT=2)
	CLOSE (UNIT=3)
	WRITE (6,*) NFILES, ' profiles processed.'
	WRITE (6,*) 'File= '//OUTFIL(1:LOUTFL)//' created.'
	WRITE (6,*) 'File=PROFPROE.CSV created.'
	CALL EXIT (0)
C
C	ERRORS
C
 902	WRITE (6,*) 'Error opening compound assignment file = '//INLIN
	GOTO 999
 903	WRITE (6,*) 'No data in a compound assignment file.'
	CLOSE (UNIT=2)
	GOTO 999
 904	WRITE (6,*) 'Error processing data in compound assignment file.'
	WRITE (6,*) 'Bad record is: ',INLIN
	CLOSE (UNIT=2)
	GOTO 999
 905	WRITE (6,*) 'Error opening SAROAD assignment file = '
     &	 //INLIN(1:LNBLNK(INLIN))
	CLOSE (UNIT=3)
	CLOSE (UNIT=4,STATUS='DELETE')
	GOTO 999
 906	WRITE (6,*) 'No data or error in SAROAD assignment file = '
     &	 //SARTYP(ISAR)(1:LSARTP)//'SARID.DAT'
 999	CALL EXIT (2)
	END
