	PROGRAM PROFPRO
C
C	PROFILE PRE-PROCESSOR
C
C	The PROFPRO program converts profile composition data with
C	chemical classes given in various categorizations to profile
C	data given in terms of specific chemical compounds using the
C	master emissions database categorization.
C
C	Use: PROFPRO InputParmFile
C	or:  PROFPRO /P:parameter input
C	
C	Where InputParmFile is the name of a file that contains the
C	parameters the program uses, and "parameter input" is a single
C	parameter line.  Default parameters are read from
C	the file MODELING.PRM, which is read before the input file.
C
C	-----------------------------------------------------------
C	Calls STSUBS subroutines
C
C	SAROAD ASSIGNMENT DATA
C
	CHARACTER SARDOC*80
	PARAMETER (MAXSAR=2000)
	CHARACTER SARNAM(MAXSAR)*12,SARID(MAXSAR)*16
	INTEGER NSARID
C
C	ASSIGNMENT OF SPECIES TO UNIQUE ID'S (UID'S)
C
	CHARACTER CMPDOC*80
	PARAMETER (MAXSPC=8000,MAXUID=3000,MAXCMP=2000)
	CHARACTER UIDLST(MAXUID)*16,UIDSPC(MAXSPC)*16,CMPID(MAXCMP)*16
	REAL SPCWT(MAXSPC)
	INTEGER CMPDNO(MAXCMP),NUID1(MAXUID),NUID2(MAXUID)
	INTEGER NUID,NSPCID
C
C	COMPOUND DESCRIPTION INFORMATION
C
	INTEGER NCMPDS
	CHARACTER DESCID(MAXCMP)*16,DESCHR(MAXCMP)*100
	LOGICAL DESCNV(MAXCMP),DESCXM(MAXCMP)
C
C	EMISSIONS FILE DATA
C
	INTEGER NCMP
	REAL TOTWT,CMPWT(MAXCMP)
C
C	LOCAL VARIABLES
C
	CHARACTER SARI*16,UID*16,OLDUID*16,CMPI*16,UID1*16
	LOGICAL SAROK
	CHARACTER EMINAME*8,EMIUIC*40,EMCUIC*40,SARTYP*4,EDBUIC*40
	CHARACTER INLIN*255,PRMFIL*80,INFILE*80,EMIFIL*60,EMCFIL*60
	CHARACTER OUTFIL*60
	LOGICAL MANY,ERROUT,NOSAR,ERRIND,MPIN
	DATA ERROUT/.FALSE./
C
C	OPEN INPUT FILE AND READ PARAMETERS
C
	CALL GETARG(1,PRMFIL)
	IF (PRMFIL(1:1).EQ.' ') THEN
		WRITE (6,*) 'Profile prepropressor to produce .EMC'
     &		 //' files giving compounds in profiles in the '
     &		 //'format required for speciation processing'
		WRITE (6,*) 'Use: PROFPRO ParameterFile'
		WRITE (6,*) ' or: PROFPRO PROF=Profile'
		WRITE (6,*) ' or: PROFPRO PROFLIST=ProfileListFile'
		CALL EXIT (2)
	ENDIF
C
	EMIUIC=" "
	EMCUIC=" "
	EDBUIC=" "
	SARTYP=" "
	INFILE=" "
	OUTFIL='PROFPRO.CSV'
	LOUTFL=11
C
C	FIRST GET PARMS FROM MODELING.PRM, THEN FROM INPUT FILE
C
	MPIN=.TRUE.
	OPEN (UNIT=1,FILE='MODELING.PRM',STATUS='UNKNOWN')
C
   10	READ (1,1,END=12) INLIN
    1	FORMAT (A)
C
C	PROFILE LIST FOR PROCESSING MULTIPLE PROFILES
C
    3	IF (INLIN(1:9).EQ.'PROFLIST=') THEN
		MANY=.TRUE.
		INFILE=INLIN(10:)
C
C	PROFILE ID FOR SINGLE PROFILE PROCESSING (USED INSTEAD OF
C	PROFLIST
C
	ELSEIF (INLIN(1:5).EQ.'PROF=') THEN
		MANY=.FALSE.
		INFILE=INLIN(6:)
C
C	LOCATION OF INPUT .EMI FILES THAT GIVE PROFILE COMPOSITIONS
C	IN THEIR NATIVE FORMAT
C
	ELSEIF (INLIN(1:8).EQ.'PROFEMI=') THEN
		EMIUIC=INLIN(9:)
		LEMIUC=LNBLNK(EMIUIC)
C
C	LOCATION OF OUTPUT .EMC FILES THAT GIVE PROFILE COMPOSITIONS
C	IN TERMS OF CHEMICAL COMPOUNDS USING SPECIATION DB DESIGNATIONS
C
	ELSEIF (INLIN(1:8).EQ.'PROFEMC=') THEN
		EMCUIC=INLIN(9:)
		LEMCUC=LNBLNK(EMCUIC)
C
C	DEFAULT NATIVE CATEGORIZATION TYPE (CAN BE OVER-WRITTEN
C	BY INPUT IN THE "PROFLIST" FILE OR BY THE SARTYPE RECORDS
C	IN THE .EMI FILES THEMSELVES.
C
	ELSEIF (INLIN(1:8).EQ.'SARTYPE=') THEN
		SARTYP=INLIN(9:)
C
C	LOCATION OF THE SPECASN.DAT FILE
C
	ELSEIF (INLIN(1:7).EQ.'EMITDB=') THEN
		EDBUIC=INLIN(8:)
		LEDBUC=LNBLNK(EDBUIC)
C
C	OUTPUT FILE IF NOT PROFPRO.CSV
C
	ELSEIF (INLIN(1:12).EQ.'PROFPRO-OUT=') THEN
		OUTFIL=INLIN(13:)
		LOUTFL=LNBLNK(OUTFIL)
C
	ENDIF
C
C	END OF PARAMETER INPUT
C
	GOTO 10
C
   12	CLOSE (UNIT=1)
	IF (MPIN) THEN
		CLOSE (UNIT=1)
		IF (PRMFIL(1:9).EQ.'PROFLIST=') THEN
			MANY=.TRUE.
			INFILE=PRMFIL(10:)
		ELSEIF (PRMFIL(1:5).EQ.'PROF=') THEN
			MANY=.FALSE.
			INFILE=PRMFIL(6:)
		ELSE
			OPEN (UNIT=1,FILE=PRMFIL,STATUS='OLD',
     &			 ERR=9001)		! read
			MPIN=.FALSE.
			GOTO 10
		ENDIF
	ENDIF
C
	IF (INFILE(1:1).EQ.' ') THEN
		WRITE (6,*) 'No PROFLIST or PROF specfied in input'
     &		 //' parameter file.'
		CALL EXIT(2)
	ELSEIF (MANY) THEN
		INLVL=1
		INUNIT=9
		OPEN (UNIT=INUNIT,FILE=INFILE,ERR=901,STATUS='OLD') ! read
		OPEN (UNIT=8,FILE=OUTFIL(1:LOUTFL),STATUS='UNKNOWN')
		IF (EMIUIC(1:1).NE.' ') WRITE (8,*)
     &		 'Native profile data read from '//EMIUIC(1:LEMIUC)
		WRITE (8,*) 'Native profile format = '//SARTYP
		IF (EMCUIC(1:1).NE.' ') WRITE (8,*)
     &		 'Profile compound data saved in '//EMCUIC(1:LEMCUC)
		IF (EDBUIC(1:1).NE.' ') WRITE (8,*)
     &		 'Profile assignments read from '//EDBUIC(1:LEDBUC)
		WRITE (8,*)
		WRITE (8,*) 'Profile,ok,Total_Mass,Mass_Input,'
     &		 //'Unkn_Vol.,Unkn_nVol.,nVol_Cmpd.,Exempts'
	ELSE
		EMINAME=INFILE
		LSNAME=LNBLNK(EMINAME)	! LNBLNK GIVES LENGTH OF STRING
	ENDIF
	NFILES=0
C
C	DETERMINE LENGTH OF FILE PREFIXES AND MAKE SURE THEY HAVE 
C	PROPER ENDING.
C
C	READ ASSIGNMENT DATA USED
C
C	READ ID-TO=COMPOUND ASSIGNMENTS
C
	IF (EDBUIC(1:1).EQ.' ') THEN
		INLIN='SPECASN.DAT'
	ELSE
		INLIN=EDBUIC(1:LEDBUC)//'SPECASN.DAT'
	ENDIF
	OPEN (UNIT=2,FILE=INLIN,STATUS='OLD',ERR=902)	! read
	READ (2,1,END=903) CMPDOC
  21	READ (2,1,END=903) INLIN
	IF (INLIN(1:1).NE.'.') GOTO 21
	NSPCID=0
	NUID=0
	OLDUID=" "
  22	READ (2,1,END=28) INLIN
	IF (INLIN(1:1).EQ.' '.OR.INLIN(1:1).EQ.'!') GOTO 22
	IF (NSPCID.EQ.MAXSPC) THEN
		WRITE (6,*) 'TOO MANY UID-TO-COMPOUND ASSIGNMENTS'
		WRITE (6,*) 'INCREASE MAXSPC IN PROFPRO.FOR'
		GOTO 904
	ENDIF
	NSPCID=NSPCID+1
	READ (INLIN,23,ERR=904) UID,UIDSPC(NSPCID),
     &	 SPCWT(NSPCID)
   23	FORMAT(2A16,F10.8)
	IF (UID.NE.OLDUID) THEN
		IF (NUID.GT.0) NUID2(NUID)=NSPCID-1
		IF (NUID.EQ.MAXUID) THEN
			WRITE (6,*) 'TOO MANY UID''S.'
			WRITE (6,*) 'INCREASE MAXUID IN EMITOEMC.FOR'
			GOTO 904
		ENDIF
		NUID=NUID+1
		UIDLST(NUID)=UID
		NUID1(NUID)=NSPCID
		OLDUID=UID
	ENDIF
	GOTO 22
   28	IF (NUID.GT.0) NUID2(NUID)=NSPCID-1
	CLOSE (UNIT=2)
C
C	READ COMPOUND DESCRIPTIONS (NOTE: DOESN'T CHECK TO SEE IF 
C	ARRAY SIZE EXCEEDED -- JUST DOESN'T GET THE DESCRIPTION, AND 
C	OUTPUTS ERROR LATER IF ANY COMPOUND DESCRIPTION IS NOT FOUND.)
C
	IF (EDBUIC(1:1).EQ.' ') THEN
		INLIN='CMPDINFO.DAT'
	ELSE
		INLIN=EDBUIC(1:LEDBUC)//'CMPDINFO.DAT'
	ENDIF
	OPEN (UNIT=2,FILE=INLIN,STATUS='OLD',ERR=902)	! read
  31	READ (2,1,END=903) INLIN
	IF (INLIN(1:1).NE.'.') GOTO 31
	NCMPDS=0
  34	IF (NCMPDS.EQ.MAXCMP) GOTO 39
	NCMPDS=NCMPDS+1
	READ (2,35,END=38) DESCID(NCMPDS),I,J,DESCHR(NCMPDS)
  35	FORMAT (A16,12X,I3,I4,2X,A)
	DESCNV(NCMPDS)=I.EQ.0
	DESCXM(NCMPDS)=J.EQ.0
	GOTO 34
  38	NCMPDS=NCMPDS-1
  39	CLOSE (UNIT=2)
C
C	DO NOT READ SAROAD ASSIGNMENTS UNTIL THEY ARE CALLED FOR
C	NOTE: THIS VERSION TREATS FILES THAT DON'T EXIST AS
C	EMPTY FILES.
C
	SAROK=.FALSE.
C
C	READ @INPUT FILE TO GET EMI FILES TO USE (IF APPLICABLE)
C
  100	IF (MANY) THEN
  101		READ (INUNIT,1,END=102) INLIN
		IF (INLIN(1:1).EQ.' '.OR.INLIN(1:1).EQ.'!') THEN
			GOTO 101
		ELSEIF (INLIN(1:1).EQ.'@') THEN
			INLVL=INLVL+1
			INUNIT=INUNIT+1
			OPEN (UNIT=INUNIT,FILE=INLIN(2:),STATUS='OLD',
     &			 ERR= 103)	! read
			GOTO 101
  103			WRITE (6,*) 'File not found for input: '
     &			 //INLIN(2:)
			INLVL=INLVL-1
			INUNIT=INUNIT-1
			GOTO 101
		ELSEIF (INLIN(1:8).EQ.'SARTYPE=') THEN
			IF (INLIN(9:12).NE.SARTYP) THEN
				SAROK=.FALSE.
				SARTYP=INLIN(9:12)
			ENDIF
			GOTO 101
		ELSE
C			THE PROFILE NAME MAY BE ON RECORDS THAT CONTAIN
C			OTHER INFORMATION. USE ONLY THE FIRST FIELD
			CALL PARSE0 (1,INLIN,80)
			CALL PARSE1 (1,EMINAME,LSNAME,8,INLIN)
			GOTO 110
		ENDIF
C
  102		CLOSE (UNIT=INUNIT)
		INLVL=INLVL-1
		IF (INLVL.EQ.0) THEN
			GOTO 800
		ELSE
			INUNIT=INUNIT-1
			GOTO 101
		ENDIF
	ENDIF
C
C	OPEN EMI FILE FOR INPUT AND EMC FILE FOR OUTPUT
C
  110	IF (EMIUIC(1:1).EQ.' ') THEN
		EMIFIL=EMINAME(1:LSNAME)//'.EMI'
		LEMIFL=LSNAME+4
	ELSE
		EMIFIL=EMIUIC(1:LEMIUC)//EMINAME(1:LSNAME)//'.EMI'
		LEMIFL=LEMIUC+LSNAME+4
	ENDIF
	IF (EMCUIC(1:1).EQ.' ') THEN
		EMCFIL=EMINAME(1:LSNAME)//'.EMC'
		LEMCFL=LSNAME+4
	ELSE
		EMCFIL=EMCUIC(1:LEMCUC)//EMINAME(1:LSNAME)//'.EMC'
		LEMCFL=LEMCUC+LSNAME+4
	ENDIF
	OPEN (UNIT=3,FILE=EMIFIL(1:LEMIFL),STATUS='OLD',ERR=194) ! read
	OPEN (UNIT=4,FILE=EMCFIL(1:LEMCFL),STATUS='UNKNOWN')
  111	READ (3,1,END=190) INLIN
	IF (INLIN(1:8).EQ.'SARTYPE=') THEN
		IF (INLIN(9:12).NE.SARTYP) THEN
			SAROK=.FALSE.
			SARTYP=INLIN(9:12)
		ENDIF
		GOTO 111
	ELSEIF (INLIN(1:7).EQ.'FORMAT=') THEN
C		IGNORE FORMAT. PARSE0 WILL BE USED.
		GOTO 111
	ELSEIF (INLIN(1:1).NE.'.') THEN
		WRITE (4,1) INLIN(1:LNBLNK(INLIN))
		GOTO 111
	ENDIF
C
C	READ SAROAD ASSIGNMENTS IF NEEDED
C
	IF (.NOT.SAROK) THEN
		IF (SARTYP(1:1).EQ.' ') THEN
			WRITE (6,*) 'Cannot process data because '
     &			 //'SARTYPE not specified.'
			GOTO 192
		ELSEIF (SARTYP.EQ.'NONE'.OR.SARTYP.EQ.'SDB ') THEN
C
C			SARTYPE="NONE" MEANS "SAROAD" ID'S ARE
C			ACTUALLY UNIQUE ID'S
C
			NOSAR=.TRUE.
			SARDOC="NONE"
		ELSE
			NOSAR=.FALSE.
			LSARTP=LNBLNK(SARTYP)
			IF (EDBUIC(1:1).EQ.' ') THEN
				INLIN=SARTYP(1:LSARTP)//'SARID.DAT'
			ELSE
				INLIN=EDBUIC(1:LEDBUC)//
     &				 SARTYP(1:LSARTP)//'SARID.DAT'
			ENDIF
			OPEN (UNIT=2,FILE=INLIN,STATUS='OLD',
     &			 ERR=905)	! read
			READ (2,1,END=905) SARDOC
 121			READ (2,1,END=906) INLIN
			IF (INLIN(1:1).NE.'.') GOTO 121
			NSARID=0
 122			READ (2,1,END=128) INLIN
			IF (INLIN(1:1).EQ.' '.OR.INLIN(1:1).EQ.'!') 
     &			 GOTO 122
			IF (NSARID.EQ.MAXSAR) THEN
  				WRITE (6,*) 'TOO MANY SAROAD '
     &				 //'ASSIGNMENTS'
  				WRITE (6,*) 'INCREASE MAXSAR IN '
     &				 //'PROFPRO.FOR'
				GOTO 906
			ENDIF
			CALL PARSE0 (2,INLIN,80)
			NSARID=NSARID+1
			CALL PARSE1 (1,SARNAM(NSARID),I,16,INLIN)
			CALL PARSE1 (2,SARID(NSARID),I,16,INLIN)
			IF (SARID(NSARID)(1:1).EQ.' ') NSARID=NSARID-1
			GOTO 122
  128			CLOSE (UNIT=2)
		ENDIF
		SAROK=.TRUE.
	ENDIF
C
	NCMP=0
	TOTWT=0.0
C
C	READ NEXT LINE OF EMI FILE AND PROCESS
C
  130	READ (3,1,END=180) INLIN
	IF (INLIN(1:1).EQ.' '.OR.INLIN(1:1).EQ.'!') GOTO 130
	CALL PARSE0 (2,INLIN,80)
	CALL PARSE1 (1,SARI,I,16,INLIN)
	CALL PARSER (2,WTI,ERRIND,INLIN)
	IF (ERRIND.OR.SARI(1:1).EQ.' '.OR.WTI.EQ.0.0) GOTO 130
C
	TOTWT=TOTWT+WTI
C
C	GET UNIQUE ID
C
	IF (NOSAR) THEN
		UID=SARI
	ELSE
		DO 132 I=1,NSARID
		IF (SARNAM(I).EQ.SARI) THEN
			UID=SARID(I)
			GOTO 134
		ENDIF
  132		CONTINUE
		WRITE (6,*) 'No assignment for SAROAD ',SARI
		IF (MANY) THEN
			WRITE (8,*) EMINAME//
     &			 ' No assignment for '//SARI
			ERROUT=.TRUE.
		ENDIF
		GOTO 192
	ENDIF
C
  134	CONTINUE
C
C	GET COMPOUND ASSIGNMENTS FOR UNIQUE ID UID.
C
	DO 140 I=1,NUID
	IF (UID.EQ.UIDLST(I)) THEN
		DO 142 J=NUID1(I),NUID2(I)
			CMPI=UIDSPC(J)
			WTJ=WTI*SPCWT(J)
			IF (NCMP.GT.0) THEN
				DO 144 K=1,NCMP
				IF (CMPI.EQ.CMPID(K)) THEN
					CMPWT(K)=CMPWT(K)+WTJ
 					GOTO 142
 				ENDIF
  144				CONTINUE
			ENDIF
			IF (NCMP.EQ.MAXCMP) THEN
				WRITE (6,*) 
     &				 'TOO MANY COMPOUNDS.'
				WRITE (6,*)
     &				 'INCREASE MAXCMP IN '
     &				 //'EMITOEMC.FOR'
				GOTO 906
			ENDIF
			NCMP=NCMP+1
			CMPID(NCMP)=CMPI
			CMPWT(NCMP)=WTJ
			DO 146 K=1,NCMPDS
			IF (CMPI.EQ.DESCID(K)) THEN
				CMPDNO(NCMP)=K
				GOTO 142
			ENDIF
  146			CONTINUE
			WRITE (6,*) 'Compound '//CMPI//' is not on '
     &			 //'compounds list in CMPDINFO.DAT'
			WRITE (6,*) 'CHECK MAXCMP IN EMITCMP.FOR'
			GOTO 906
  142		CONTINUE
		GOTO 130
	ENDIF
  140	CONTINUE
	WRITE (6,*) 'No compound assignment for UID ',UID
	IF (MANY) THEN
		WRITE (8,*) EMINAME//' No assignment for UID '//UID
		ERROUT=.TRUE.
	ENDIF
	GOTO 192
C
C	NORMAL END OF EMI FILE INPUT.  OUTPUT DATA
C	DATA SORTED FIRST BY AMOUNT, THEN BY UID NAME.
C
  180	IF (TOTWT.EQ.0.0) GOTO 190
	CLOSE (UNIT=3)
	TOTWT2=0.0
	UNAWT=0.0
	UNVWT=0.0
	XVCWT=0.0
	XMCWT=0.0
	DO 182 I=1,NCMP
	WTI=CMPWT(I)
	IF (CMPID(I)(1:6).EQ.'NOASN ') THEN
		UNAWT=UNAWT+WTI
	ELSEIF (CMPID(I)(1:7).EQ.'NONVOL ') THEN
		UNVWT=UNVWT+WTI
	ELSE
		NOI=CMPDNO(I)
		IF (DESCNV(NOI)) XVCWT=XVCWT+WTI
		IF (DESCXM(NOI)) XMCWT=XMCWT+WTI		
	ENDIF
  182	TOTWT2=TOTWT2+WTI
C
	WRITE (4,181) SARDOC(1:LNBLNK(SARDOC)),
     &	 CMPDOC(1:LNBLNK(CMPDOC)),TOTWT2
  181	FORMAT ('FORMAT=(A16,1X,G12.6,I3,I4,2X,A)'/
     &	 'SAR_ASSIGN=',A/'CMPD_ASSIGN=',A/'TOTWT=',1PE12.6)
	WRITE (INLIN(1:12),1813) TOTWT
	WRITE (INLIN(13:24),1813) TOTWT2
 1813	FORMAT(1PE12.6)
	IF (INLIN(1:12).NE.INLIN(13:24)) WRITE (4,1814) TOTWT
 1814	FORMAT ('TOTWT_INPUT=',1PE12.6)
	IF (UNAWT.NE.0.0) WRITE (4,1811) 'UNKNOWN',UNAWT/TOTWT
	IF (UNVWT.NE.0.0) WRITE (4,1811) 'UNK_NVOL',UNVWT/TOTWT
	UNVT=UNVWT+XVCWT
	IF (UNVT.NE.0.0) WRITE (4,1811) 'NONVOL',UNVT/TOTWT
	IF (XMCWT.NE.0.0) WRITE (4,1811) 'EXEMPT',XMCWT/TOTWT
	WRITE (4,1812)
 1811	FORMAT ('FRCT_',A,'=',F6.4)
 1812	FORMAT ('.'/"!UID_CODE       WT_FRACT     VOL VOC DESCRIPTION")
C
	DO 184 I=1,NCMP
	UID=CMPID(I)
	WTI=CMPWT(I)
	NOI=CMPDNO(I)
	IF (I.LT.NCMP) THEN
		DO 185 J=I+1,NCMP
		IF (CMPWT(J).GT.WTI.OR.(CMPWT(J).EQ.WTI.AND.
     &		 CMPID(J).GT.UID)) THEN
			UID1=CMPID(J)
			WTI1=CMPWT(J)
			NOI1=CMPDNO(J)
			CMPID(J)=UID
			CMPWT(J)=WTI
			CMPDNO(J)=NOI
			UID=UID1
			WTI=WTI1
			NOI=NOI1
		ENDIF
  185		CONTINUE
	ENDIF
	INLIN=DESCHR(NOI)
	IF (DESCNV(NOI)) THEN
		I1=0
	ELSE
		I1=1
	ENDIF
	IF (DESCXM(NOI)) THEN
		I2=0
	ELSE
		I2=1
	ENDIF
  184	WRITE (4,186) UID,WTI,I1,I2,INLIN(1:LNBLNK(INLIN))
  186	FORMAT (A16,1PG12.6,I3,I4,"  ",A)
	CLOSE (UNIT=4)
	IF (MANY) WRITE (8,188) EMINAME,TOTWT2,TOTWT,UNAWT,UNVWT,
     &	 XVCWT,XMCWT
  188	FORMAT (A8,',ok',6(',',1PE10.4))
	NFILES=NFILES+1
C
	GOTO 200
C
C	ERROR PROCESSING DATA FOR THE INPUT .EMI FILE
C
  190	CLOSE (UNIT=3)
	CLOSE (UNIT=4,STATUS='DELETE')
	WRITE (6,*) 'File = '//EMIFIL(1:LEMIFL)//
     &	 ' has no data.'
	IF (MANY) WRITE (8,*) EMINAME//' No data in file.'
	GOTO 200
C
  192	CLOSE (UNIT=3)
	CLOSE (UNIT=4,STATUS='DELETE')
	WRITE (6,*) 'Error processing data in File = '
     &	 //EMIFIL(1:LEMIFL)
	IF (MANY) THEN
		IF (ERROUT) THEN
			ERROUT=.FALSE.
		ELSE
			WRITE (8,*) EMINAME//' Error processing data.'
		ENDIF
	ENDIF
	GOTO 200
C
  194	WRITE (6,*) 'File = '//EMIFIL(1:LEMIFL)//' does not exist.'
	IF (MANY) WRITE (8,*) EMINAME//' No file.'
C
C	NEXT EMI FILE IF APPLICABLE
C
  200	IF (MANY) GOTO 100
C
C	END OF INPUT.
C
  800	IF (NFILES.EQ.0) THEN
		WRITE (6,*) 'No files processed.'
	ELSEIF (NFILES.EQ.1) THEN
		WRITE (6,*) 'File = '//EMCFIL(1:LEMCFL)//
     &		 ' created.'
	ELSE
		WRITE (6,*) NFILES, ' files processed.'
	ENDIF
	IF (MANY) THEN
		CLOSE (UNIT=8)
		WRITE (6,*) 'File= '//OUTFIL(1:LOUTFL)//' created.'
	ENDIF
	CALL EXIT (0)
C
C	ERRORS
C
 9001	WRITE (6,*) 'Error opening input file= '//PRMFIL
	GOTO 999
 901	WRITE (6,*) 'Error opening file= '//INFILE
	GOTO 999
 902	WRITE (6,*) 'Error opening compound assignment file = '//INLIN
	GOTO 999
 903	WRITE (6,*) 'No data in a compound assignment file.'
	CLOSE (UNIT=2)
	GOTO 998
 904	WRITE (6,*) 'Error processing data in compound assignment file.'
	WRITE (6,*) 'Bad record is: ',INLIN
	CLOSE (UNIT=2)
	GOTO 999
 905	WRITE (6,*) 'Error opening SAROAD assignment file = '
     &	 //INLIN(1:LNBLNK(INLIN))
	CLOSE (UNIT=3)
	CLOSE (UNIT=4,STATUS='DELETE')
	GOTO 998
 906	WRITE (6,*) 'No data or error in SAROAD assignment file = '
     &	 //SARTYP(1:LSARTP)//'SARID.DAT'
	CLOSE (UNIT=2)
	CLOSE (UNIT=3)
	CLOSE (UNIT=4,STATUS='DELETE')
	GOTO 998
 998	IF (MANY) THEN
		CLOSE (UNIT=1)
		CLOSE (UNIT=8)
	ENDIF
 999	CALL EXIT (2)
	END	
