	PROGRAM SPECPRO
C
C	SPECIATION PREPROCESSOR
C
C	The SPECPRO program processes specified profiles
C	for a specified mechanism and outputs a file
C	containing model species assigned to those profiles in the
C	format required by the SMOKE system. It can also
C	optionally output a file giving the VOC to TOG conversion
C	factors based on the fraction of compounds in the profile
C	that are on the EPA exempt list. 
C	
C	Use: SPECPRO InputParmFile
C	Or:  SPECPRO PROFLIST=file
C	Or:  SPECPRO PROF=profile [mech] [cmpfile]
C
C	where InputParmFile is the input file containing the
C	parameters controlling the program, or
C	paramerter1, etc., are parameter input lines to be processed
C	along with those in MODELING.PRM.
C
C	Calls RDASN to read compound and mechanism assignments
C	CALLS RDPRO to read and proces procfile data
C	Calls STSUBS subroutines
C
C	FILE NAMES AND GLOBAL PARAMETERS
C
	CHARACTER GSPFIL*60,EMCUIC*40,UNKEMC*60,UNAEMC*60
	CHARACTER MRGFL1*60,MRGFL2*60,OUTFIL*60,TOGVOC*60,PRMFIL*80
	CHARACTER CMPOPT*1
	LOGICAL VOCFAC
C
C	SPECIFICATIONS FOR MODEL SPECIES AND COMPOUNDS FOR
C	SPECPRO AND ITS SUBROUTINES
C
	INCLUDE 'SPECSPEC.FOR'
C
C	PROFILE DATA
C
	REAL TOTMOL,TOTMAS,UNKMAS,UNAMAS,UNAMOL,UNRMAS,NOVMAS,TOTNC,
     &	 VOLMAS,MSMAS(MAXMS),MSMOL(MAXMS),XMTMAS
	COMMON /PRODAT/ TOTMAS,TOTMOL,UNKMAS,UNAMAS,UNAMOL,UNAC,
     &	 UNRMAS,NOVMAS,TOTNC,VOLMAS,MSMAS,MSMOL,XMTMAS
C
C
C	SPECIAL PROFILES (CAN'T HAVE UNKNOWN OR UNASSIGNED MASS)
C
	REAL MSUNKM(MAXMS),MSUNKO(MAXMS),TUNKO,TUNKC ! ,XMTUNK
	REAL MSNOAM(MAXMS),MSNOAO(MAXMS),TNOAO,TNOAC ! ,XMTNOA
C
C	PROCESSED PROFILE LOG
C
	PARAMETER (MAXPRO=2000)
	CHARACTER PROID(MAXPRO)*14,PROIN*14
	INTEGER NPRO
C
C	LOCAL ASSIGNMENTS
C
	CHARACTER INLIN*250,INFILE*80,PNUM*8,PTYP*8,NAMEIN*8
	CHARACTER PROFIL*60,PROCHR*60,CMPUIC*60,CMPFIL*60
	INTEGER OUTFOR,UNKIND,UNAIND
	REAL LOSTC,LOSTM
	LOGICAL ERRIND,OUTOK,STDOUT,ONEPRO,MPIN,MLTPRO
C
C	COMMENT OR UN-COMMENT AS APPROPRIATE
C
	CHARACTER FILDLM/'\\'/	! PC SYSTEMS
C	CHARACTER FILDLM/'/'/	! UNIX SYSTEMS
C
C	OPEN INPUT FILE AND READ PARAMETERS
C
	CALL GETARG (1,PRMFIL)			! GET COMMAND LINE
	IF (PRMFIL(1:1).EQ.' ') THEN
		WRITE (6,*) 'Speciation prepropressor to produce '
     &		 //'GSPRO files for a specified mechansim and'
		WRITE (6,*) 'set of profiles'
		WRITE (6,*) 'Use: SPECPRO inputfile'
		WRITE (6,*) ' or: SPECPRO PROFLIST=file'
		WRITE (6,*) ' or: SPECPRO PROF=Profile Mech'
		CALL EXIT (2)
	ENDIF
C
	OUTFOR=1	! GSPRO OUTPUT FORMATS: 0=CSV, 1=SMOKE
	EMCUIC=' '
	EDBUIC=' '
	UNKEMC=' '
	UNAEMC=' '
	PROLFL=' '
	MECHID=' '
	GSPFIL=' '
	MRGFL1=' '
	MRGFL2=' '
	OUTFIL=' '
	TOGVOC=' '
	PROFIL=' '
	CMPFIL=' '
	CMPOPT=' '
	VOCFAC=.FALSE.
	XINFO=.FALSE.
	STDOUT=.TRUE.
	MLTPRO=.FALSE.
C
C	FIRST GET PARMS FROM MODELING.PRM, THEN FROM INPUT FILE
C
	MPIN=.TRUE.
	OPEN (UNIT=1,FILE='MODELING.PRM',STATUS='UNKNOWN')
C
   10	READ (1,1,END=12) INLIN
    1	FORMAT (A)
    3	IF (INLIN(1:9).EQ.'PROFLIST=') THEN
		PROLFL=INLIN(10:)
		LPROLF=LNBLNK(PROLFL)		! STRING LENGTH
		ONEPRO=.FALSE.
	ELSEIF (INLIN(1:8).EQ.'ALLPROF=') THEN
		PROLFL=INLIN(9:)
		LPROLF=LNBLNK(PROLFL)
		ONEPRO=.FALSE.
		MLTPRO=.TRUE.
	ELSEIF (INLIN(1:5).EQ.'MECH=') THEN
		MECHID=INLIN(6:)
		LMECID=LNBLNK(MECHID)
	ELSEIF (INLIN(1:6).EQ.'GSPRO=') THEN
		CALL PARSE0 (2,INLIN(7:),80)
		CALL PARSE1 (1,GSPFIL,LGSPFL,60,INLIN(7:))
		CALL PARSE2 (2,NAMEIN,I,8,INLIN(7:))
		IF (NAMEIN(1:4).EQ.'CSV ') THEN
			OUTFOR=0
		ELSEIF (NAMEIN(1:6).EQ.'SMOKE ') THEN
			OUTFOR=1
		ELSEIF (NAMEIN(1:1).NE.' ') THEN
			WRITE (6,*) 'Unrecognized format: '//
     &			 INLIN(7:LNBLNK(INLIN))
			CALL EXIT (2)
		ENDIF
	ELSEIF (INLIN(1:8).EQ.'PROFEMC=') THEN
		EMCUIC=INLIN(9:)
		LEMCUC=LNBLNK(EMCUIC)
	ELSEIF (INLIN(1:7).EQ.'EMITDB=') THEN
		EDBUIC=INLIN(8:)
		LEDBUC=LNBLNK(EDBUIC)
	ELSEIF (INLIN(1:12).EQ.'SPECPRO-OUT=') THEN
		OUTFIL=INLIN(13:)
		LOUTFL=LNBLNK(OUTFIL)
	ELSEIF (INLIN(1:9).EQ.'UNKN-ASN=') THEN
		UNKEMC=INLIN(10:)
	ELSEIF (INLIN(1:10).EQ.'UNASN-ASN=') THEN
		UNAEMC=INLIN(11:)
	ELSEIF (INLIN(1:9).EQ.'GSPRO-IN=') THEN
		IF (MRGFL1(1:1).EQ.' ') THEN
			MRGFL1=INLIN(10:)
			LMRGF1=LNBLNK(MRGFL1)
		ELSEIF (MRGFL2(1:1).EQ.' ') THEN
			MRGFL2=INLIN(10:)
			LMRGF2=LNBLNK(MRGFL2)
		ELSE
			WRITE (6,*) 'No more than 2 GSPRO-IN files '
     &			 //'can be processed.'
			CALL EXIT (2)
		ENDIF
	ELSEIF (INLIN(1:11).EQ.'TOGVOC-FAC=') THEN
		TOGVOC=INLIN(12:)
		IF (TOGVOC(1:1).NE.' ') THEN
			LTOGVC=LNBLNK(TOGVOC)
			VOCFAC=.TRUE.
		ENDIF
	ELSEIF (INLIN(1:6).EQ.'DEBUG ') THEN
		XINFO=.TRUE.
	ELSEIF (INLIN(1:8).EQ.'PROFCMP=') THEN
		CMPUIC=INLIN(9:)
		LCMPUC=LNBLNK(CMPUIC)
	ELSEIF (INLIN(1:5).EQ.'PROF=') THEN
		CALL PARSE0 (3,INLIN(6:),74)
		CALL PARSE1 (1,PROFIL,LPROFL,8,INLIN(6:))
		CALL PARSE1 (2,NAMEIN,I,8,INLIN(6:))
		CALL PARSE1 (3,CMPFIL,LCMPFL,60,INLIN(6:))
		IF (NAMEIN(1:1).NE.' ') THEN
			MECHID=NAMEIN
			LMECID=I
		ENDIF
		ONEPRO=.TRUE.
		STDOUT=.FALSE.
	ELSEIF (INLIN(1:15).EQ.'SPECPRO=CMPMODE') THEN
		STDOUT=.FALSE.
	ELSEIF (INLIN(1:8).EQ.'CMPOPT=C') THEN
		CMPOPT='C'
	ELSEIF (INLIN(1:8).EQ.'CMPOPT=M') THEN
		CMPOPT='M'
	ENDIF
C	END PARAMETER INPUT
C
	GOTO 10
C
C	<EOF> ON INPUT UNIT. UNIT KEPT OPEN TO RE-READ TO GET
C	MECHANISM OVERRIDE ASSIGNMENTS AFTER DEFAULT ASSIGNMENTS READ
C	IN.
C
   12	CONTINUE
	IF (MPIN) THEN
		IF (PRMFIL(1:9).EQ.'PROFLIST=') THEN
			PROLFL=PRMFIL(10:)
			LPROLF=LNBLNK(PROLFL)
			ONEPRO=.FALSE.
		ELSEIF (PRMFIL(1:5).EQ.'PROF=') THEN
			PROFIL=PRMFIL(6:)
			LPROFL=LNBLNK(PROFIL)
			CALL GETARG (2,NAMEIN)
			IF (NAMEIN(1:1).NE.' ') THEN
				MECHID=NAMEIN
				LMECID=LNBLNK(MECHID)
			ENDIF
			CALL GETARG(3,CMPFIL)
			LCMPFL=LNBLNK(CMPFIL)
			ONEPRO=.TRUE.
			STDOUT=.FALSE.
		ELSE
			CLOSE (UNIT=1)
			OPEN (UNIT=1,FILE=PRMFIL,STATUS='OLD',
     &			 ERR=9001)	! READONLY
			MPIN=.FALSE.
			GOTO 10
		ENDIF
	ENDIF
C
	ERRIND=.FALSE.
	IF (MECHID(1:1).EQ.' ') THEN
		WRITE (6,*) 'MECH not specified'
		ERRIND=.TRUE.
	ENDIF
	IF (ONEPRO) THEN
		IF (PROFIL(1:1).EQ.' ') THEN
			WRITE (6,*) 'PROF not specified.'
			ERRIND=.TRUE.
		ENDIF
	ELSEIF (PROLFL(1:1).EQ.' ') THEN
		WRITE (6,*) 'PROFLIST or ALLPROF not specified.'
		ERRIND=.TRUE.
	ENDIF
	IF (STDOUT.AND.GSPFIL(1:1).EQ.' ') THEN
		WRITE (6,*) 'GSPRO not specified.'
		ERRIND=.TRUE.
	ENDIF
	IF (ERRIND) CALL EXIT (2)
C
	IF (.NOT.STDOUT) THEN
		XINFO=.FALSE.
		OUTFOR=-1
	ENDIF
C
C	DONE PROCESSING PARAMETERS
C
C	READ COMPOUND AND MECHANISM ASSIGNMENTS USING RDASN SUBROUTINE
C
	REWIND (UNIT=1)
	CALL RDASN (1,.FALSE.)
	CLOSE (UNIT=1)
C
C	OPEN "EXTRA INFORMATION" OUTPUT FILE FOR DEBUG PARAMETER
C
	IF (XINFO) OPEN (UNIT=10,FILE='SPECPRO1.CSV',STATUS='UNKNOWN')
C
C	OPEN TOG/VOC FACTOR OUTPUT FILE IF SPECIFIED
C
	IF (VOCFAC) OPEN (UNIT=9,FILE=TOGVOC(1:LTOGVC),
     &	 STATUS='UNKNOWN')
C
C	----------------------------------------------------
C	ASSIGNMENT DATA ARE NOW READ IN.
C	----------------------------------------------------
C
	IF (STDOUT) THEN
		IF (OUTFIL.EQ.' ') THEN
 			OUTFIL='SPECPRO.CSV'
			LOUTFL=11
		ENDIF
C
		OPEN (UNIT=3,FILE=OUTFIL,STATUS='UNKNOWN')
		IOUT=3
	ELSE
		IOUT=0
	ENDIF
C
C	DEFINE THE UNKNOWN MASS AND UNASSIGNED COMPOUND PROFILES
C	NORMALIZE TO TOTMAS=1.
C	*NOTE* CODE TO COMPUTE EXEMPT MASS COMMENTED OUT SINCE
C	THIS IS NOT COUNTED FOR UNKNOWN OR UNASSIGNED PROFILES
C
	IF (UNKEMC(1:1).EQ.' ') THEN
		UNKIND=0
	ELSEIF (UNKEMC(1:5).EQ.'NORM ') THEN
		UNKIND=-1
	ELSEIF (UNKEMC(1:6).EQ.'INERT ') THEN
		IF (INROG.NE.0) THEN
			UNKIND=-2
		ELSE
			UNKIND=0
		ENDIF
	ELSEIF (UNKEMC(1:5).EQ.'NONE ') THEN
		UNKIND=0
	ELSE
		UNKIND=1
		IF (EMCUIC(1:1).EQ.' ') THEN
			INFILE=UNKEMC(1:LNBLNK(UNKEMC))//'.EMC'
		ELSE
			INFILE=EMCUIC(1:LEMCUC)//
     &			 UNKEMC(1:LNBLNK(UNKEMC))//'.EMC'
		ENDIF
		CALL RDPRO (INFILE,IOUT,0)
		IF (UNKMAS.NE.0.0.OR.UNAMAS.NE.0.0.OR.NOVMAS.NE.0.0
     &		 .OR.UNRMAS.NE.0.0.OR.TOTMAS.EQ.0.0) THEN
			WRITE (6,*) 'Unknown, unassigned or non-'//
     &			 'volatile mass not allowed in unknown '//
     &			 'cmpd. profile.'
			WRITE (6,*) 'Profile file = '//INFILE
			WRITE (6,*) 'Unknown=',UNKMAS
			WRITE (6,*) 'Unasn=',UNAMAS
			WRITE (6,*) 'NonVol=',NOVMAS
			CALL EXIT (2)
		ENDIF
		DO 52 I=1,NMS
		MSUNKO(I)=MSMOL(I)/TOTMAS
   52		MSUNKM(I)=MSMAS(I)/TOTMAS
		TUNKO=TOTMOL/TOTMAS
		TUNKC=TOTNC/TOTMAS
C		XMTUNK=XMTMAS/TOTMAS
 	ENDIF
C
	IF (UNAEMC(1:1).EQ.' ') THEN
		UNAIND=0
	ELSEIF (UNAEMC(1:5).EQ.'NORM ') THEN
		UNAIND=-1
	ELSEIF (UNAEMC(1:6).EQ.'INERT ') THEN
		IF (INROG.NE.0) THEN
			UNAIND=-2
		ELSE
			UNAIND=0
		ENDIF
	ELSEIF (UNAEMC(1:5).EQ.'NONE ') THEN
		UNAIND=0
	ELSEIF (UNAEMC.EQ.UNKEMC) THEN
		UNAIND=3
		DO 5111 I=1,NMS
		MSNOAO(I)=MSUNKO(I)
 5111		MSNOAM(I)=MSUNKM(I)
		TNOAO=TUNKO
		TNOAC=TUNKC
C		XMTNOA=XMTUNK
	ELSE
		UNAIND=3
		IF (EMCUIC(1:1).EQ.' ') THEN
			INFILE=UNAEMC(1:LNBLNK(UNAEMC))//'.EMC'
		ELSE
			INFILE=EMCUIC(1:LEMCUC)
     &			 //UNAEMC(1:LNBLNK(UNAEMC))//'.EMC'
		ENDIF
		CALL RDPRO (INFILE,IOUT,0)
		IF (UNKMAS.NE.0.0.OR.UNAMAS.NE.0.0.OR.NOVMAS.NE.0.0
     &		 .OR.UNRMAS.NE.0.0.OR.TOTMAS.EQ.0.0) THEN
			WRITE (6,*) 'Unknown, unassigned or non-'//
     &			 'volatile mass not allowed in unassigned'//
     &			 ' cmpd. profile.'
			WRITE (6,*) 'Profile file = '//INFILE
			CALL EXIT (2)
		ENDIF
		DO 521 I=1,NMS
		MSNOAO(I)=MSMOL(I)/TOTMAS
  521		MSNOAM(I)=MSMAS(I)/TOTMAS
		TNOAO=TOTMOL/TOTMAS
		TNOAC=TOTNC/TOTMAS
C		XMTNOA=XMTMAS/TOTMAS
	ENDIF
C
C
C	PROCESS PROFILES.  IF MULTIPLE ASSIGNMENTS ARE GIVEN FOR A
C	PROFILE (IDENTIFIED BY NUMBER AND TYPE COMBO), ONLY
C	THE *FIRST* IS USED.
C	OPEN THE OUTPUT GSPRO FILE
C
	NPRO=0
	IF (ONEPRO) THEN
		INU=0
	ELSE
		OPEN (UNIT=1,FILE=PROLFL,STATUS='OLD',
     &		 ERR=901)	! READONLY
		INU=1
	ENDIF
	IF (STDOUT) THEN
		OPEN (UNIT=2,FILE=GSPFIL,STATUS='UNKNOWN')
		WRITE (3,1) MECHID(1:LMECID)
		IF (UNKIND.EQ.0) THEN
			WRITE (3,1) 'Unknowns ignored '
     &			 //'profile not re-normalized'
		ELSEIF (UNKIND.EQ.-2) THEN
			WRITE (3,1) 'Unknowns treated as INERT'
		ELSEIF (UNKIND.EQ.-1) THEN
			WRITE (3,1) 'Unknowns represented by rest of '
     &			 //'profile'
		ELSE
			WRITE (3,1) 'Unknowns represented by '//
     &			 UNKEMC(1:LNBLNK(UNKEMC))//'.EMC'
		ENDIF
		IF (UNAIND.EQ.0) THEN
			WRITE (3,1) 'Unassigned VOCs ignored (not '
     &			 //'renormalized'
		ELSEIF (UNAIND.EQ.-2) THEN
			WRITE (3,1) 'Unassigned VOCs treated as INERT'
		ELSEIF (UNAIND.EQ.-1) THEN
			WRITE (3,1) 'Unassigned VOCs represented by '
     &			 //'rest of profile'
		ELSE
			WRITE (3,1) 'Unassigned VOCs represented by '
     &			 //UNAEMC(1:LNBLNK(UNAEMC))//'.EMC'
		ENDIF
 		WRITE (3,1) 'Num,Type,File,Msg,Tot Mass,VOL Mass,VOC '
     &		 //'Mass,Avg Mwt,Tot Mol,Unk Mass,Una''d Mass,Una''d '
     &		 //'Mol,Inert,Non VOL,nC,MWt/C,Lost C,Lost Mass'
C
		IF (OUTFOR.EQ.0) THEN
		    NMSOUT=MIN(NMS,64)
		    IF (NMSOUT.LT.NMS) THEN
			WRITE (6,*) 'Warning: last ',NMS-NMSOUT,
     &			 ' model species could not be output.'
			WRITE (6,*) 'No more than 64 model species '
     &			 //'can be output.'
		    ENDIF
		    WRITE (2,1001) MECHID(1:LMECID),
     &		     (MSNAM(I)(1:LNBLNK(MSNAM(I))),I=1,NMS)
 1001		    FORMAT(A/'Num,Type,File,Mass,VOL,VOC,Avg ',
     &		     'Mw,Unkn,Unasn,Unrct,Non VOL',64(',',A))
		ENDIF
	ENDIF
C
C	-----------------------------------------------------
C	MAIN LOOP OVER PROFILES TO BE ASSIGNED MODEL SPECIES,
C	USING FILES LISTED IN INPUT FILE=PROLFILE.
C
C	ALL PROFILES ARE PROCESSED AS "TOG" PROFILES.
	PTYP='TOG'
	LPTYP=3
	IF (ONEPRO) GOTO 199
	IF (MLTPRO) THEN
 1002		READ (1,1,END=800) INLIN
		IF (INLIN(1:2).NE.'..') GOTO 1002
	ENDIF
C
  100	READ (1,1,END=800) INLIN
	IF (INLIN(1:1).EQ.' '.OR.INLIN(1:1).EQ.'!') GOTO 100
	IF (MLTPRO) THEN
		PROFIL=INLIN
		LPROFL=LNBLNK(PROFIL)
		CALL RDPRO (PROFIL,IOUT,INU)
		PROIN=PROFIL
		PROCHR=PROFIL(1:LPROFL)//',,'
		LPROCH=LPROFL+2
		NPRO=NPRO+1
		GOTO 1991
	ENDIF
C
	CALL PARSE0 (2,INLIN,80)
	CALL PARSE1 (1,PROFIL,LPROFL,60,INLIN)
	CALL PARSE1 (2,PNUM,LPNUM,8,INLIN)
	IF (PNUM(1:1).EQ.' ') THEN
		IF (LPROFL.LE.4) THEN
			PNUM=PROFIL
			LPNUM=LPROFL
		ELSE
			PNUM=PROFIL(LPROFL-3:LPROFL)
			LPNUM=4
		ENDIF
	ENDIF
C
C	IGNORE IF ALREADY PROCESSED
C
	PROIN=PNUM//'  '//PTYP
	PROCHR=PNUM(1:LPNUM)//','//PTYP(1:LPTYP)//','//
     &	 PROFIL(1:LPROFL)
	LPROCH=LPNUM+LPTYP+LPROFL+2
	IF (NPRO.GT.0) THEN
		DO 101 I=1,NPRO
		IF (PROIN.EQ.PROID(I)) THEN
			IF (STDOUT) WRITE (3,1) 
     &			 PROCHR(1:LPROCH)//',Skipped'
			GOTO 100
		ENDIF
  101		CONTINUE
	ENDIF
	IF (NPRO.EQ.MAXPRO) THEN
		WRITE (6,*) 'INCREASE MAXPRO IN SPECPRO.FOR'
		CALL EXIT (2)
	ENDIF
	NPRO=NPRO+1
	PROID(NPRO)=PNUM//'  '//PTYP
C
C	OPEN PROFILE SPECIATION FILE (PROFIL.EMC) AND PROCESS PROFILE
C	-------------------------------------------------------------
  199	IF (EMCUIC(1:1).EQ.' ') THEN
		INFILE=PROFIL(1:LPROFL)//'.EMC'
	ELSE
		INFILE=EMCUIC(1:LEMCUC)//PROFIL(1:LNBLNK(PROFIL))
     &		 //'.EMC'
	ENDIF
	CALL RDPRO(INFILE,IOUT,0)
C	-------------------------------------------------------------
 1991	IF (TOTMAS.EQ.0.0) THEN
		IF (STDOUT) WRITE (3,1) PROCHR(1:LPROCH)//',error'
		IF (ONEPRO) CALL EXIT (2)
		GOTO 100
	ENDIF
C
	IF (XINFO.AND.(OUTFOR.EQ.1)) WRITE (2,1)
C
C	MERGE UNASSIGNED OR UNKNOWN MASS, IF PROFILES FOR THEM 
C	DEFINED
C	*NOTE* CODE TO COMPUTE EXEMPT MASS COMMENTED OUT SINCE
C	THIS IS NOT COUNTED FOR UNKNOWN OR UNASSIGNED PROFILES
C
	IF (UNKMAS.NE.0.0) THEN
		IF (UNKIND.EQ.-2) THEN
C			ALREADY MADE SURE INROG NE 0.
			MSMAS(INROG)=MSMAS(INROG)+UNKMAS
			UNKMOL=UNKMAS/MSMWT(INROG) !USE NR MWT
			MSMOL(INROG)=MSMOL(INROG)+UNKMOL
			TOTMOL=TOTMOL+UNKMOL
			VOLMAS=VOLMAS+UNKMAS
		ELSEIF (UNKIND.EQ.-1) THEN
			IF (VOLMAS.NE.0.0) THEN
				FAC=(VOLMAS+UNKMAS)/VOLMAS
				DO 5911 I=1,NMS
				MSMAS(I)=MSMAS(I)*FAC
 5911				MSMOL(I)=MSMOL(I)*FAC
				TOTMOL=TOTMOL*FAC
				VOLMAS=VOLMAS*FAC
C				XMTMAS=XMTMAS*FAC
C			ELSE
C				NOT PROCESSED - VOLMASS REMAINS 0
			ENDIF
		ELSEIF (UNKIND.NE.0) THEN
			DO 591 I=1,NMS
			MSMAS(I)=MSMAS(I)+(UNKMAS*MSUNKM(I))
  591			MSMOL(I)=MSMOL(I)+(UNKMAS*MSUNKO(I))
			TOTMOL=TOTMOL+(UNKMAS*TUNKO)
			TOTNC=TOTNC+(UNKMAS*TUNKC)
C			XMTMAS=XMTMAS+(UNKMAS*XMTUNK)
			VOLMAS=VOLMAS+UNKMAS
C		ELSEIF (UNKIND.EQ.0) THEN
C			NOT PROCESSED
		ENDIF
	ENDIF
C
	IF (UNAMOL.NE.0.0) THEN
		IF (UNAIND.EQ.-2) THEN
C			(INROG.EQ.0) already checked for
			MSMAS(INROG)=MSMAS(INROG)+UNAMAS
			MSMOL(INROG)=MSMOL(INROG)+UNAMOL
			TOTMOL=TOTMOL+UNAMOL
			VOLMAS=VOLMAS+UNAMAS
		ELSEIF (UNAIND.EQ.-1) THEN
			IF (VOLMAS.NE.0.0) THEN
				FAC=(VOLMAS+UNAMAS)/VOLMAS
				DO 5912 I=1,NMS
				MSMAS(I)=MSMAS(I)*FAC
 5912				MSMOL(I)=MSMOL(I)*FAC
				TOTMOL=TOTMOL*FAC
				VOLMAS=VOLMAS*FAC
C				XMTMAS=XMTMAS*FAC
C			ELSE
C				NOT PROCESSED - VOLMASS REMAINS 0
			ENDIF
		ELSEIF (UNAIND.NE.0) THEN
			DO 592 I=1,NMS
			MSMAS(I)=MSMAS(I)+(UNAMAS*MSNOAM(I))
  592			MSMOL(I)=MSMOL(I)+(UNAMAS*MSNOAO(I))
			TOTMOL=TOTMOL+(UNAMAS*TNOAO)-UNAMOL
			TOTNC=TOTNC+(UNAMAS*TNOAC)-UNAC
C			XMTMAS=XMTMAS+(UNAMAS*XMTNOA)
			VOLMAS=VOLMAS+UNAMAS
C		ELSEIF (UNAIND.EQ.0) THEN
C			NOT PROCESSED
		ENDIF
	ENDIF
C
C	INERTS SHOULD ALREADY MERGED INTO THE "INERT" MODEL SPECIES,
C	IF ANY.  IF NO INERT MODEL SPECIES THEY ARE IGNORED (TREATED
C	LIKE NONVOL)
C
C	NONVOLS ARE ASSUMED NOT TO BE PRESENT AND ARE IGNORED.
C
C	OUTPUT DATA.
C
C	IF NO MOLES, OUTPUT ZERO MOLES OF UNREACTIVE MODEL
C	SPECIES (OR FIRST MODEL SPECIES IF NONE ARE UNREACTIVE)
C	AND OUTPUT MASS THAT IS NOT NONVOL
C
	LOSTM=TOTMAS-NOVMAS
	LOSTC=TOTNC
	VOCMAS=TOTMAS-XMTMAS
	IF (TOTMOL.EQ.0.0) THEN
		IF (OUTFOR.EQ.1) THEN
			WRITE (2,202) PROIN,MSNAM(MAX(INROG,1)),0.0,
     &			 1.0,TOTMAS-NOVMAS
		ELSEIF (OUTFOR.EQ.0) THEN
			WRITE (2,1) PROCHR(1:LPROCH)//',No moles'
		ELSE
			WRITE (6,*) 'Profile '//PROFIL(1:LPROFL)//
     &			 ' has no moles. Not processed.'
		ENDIF
		AVGMW=0.0
	ELSE
		AVGMW=VOLMAS/TOTMOL	! AVG MWT.
		IF (STDOUT.AND.OUTFOR.EQ.1) THEN
			DO 201 I=1,NMS
			IF (MSMOL(I).NE.0.0) WRITE (2,202) PROIN,
     &			 MSNAM(I),AVGMW*MSMOL(I),AVGMW,MSMAS(I)
			LOSTC=LOSTC-(MSMOL(I)*FLOAT(MSNC(I)))
			LOSTM=LOSTM-(MSMOL(I)*MSMWT(I))
  201			CONTINUE
		ELSEIF (STDOUT.AND.OUTFOR.EQ.0) THEN
			WRITE (2,2021) PROCHR(1:LPROCH),TOTMAS,
     &			 VOLMAS,TOTMAS-NOVMAS,AVGMW,UNKMAS,UNAMAS,
     &			 UNRMAS,NOVMAS,(MSMOL(I),I=1,NMSOUT)
			DO 2011 I=1,NMS
			LOSTC=LOSTC-(MSMOL(I)*FLOAT(MSNC(I)))
			LOSTM=LOSTM-(MSMOL(I)*MSMWT(I))
 2011			CONTINUE
		ELSE
			IF (CMPFIL(1:1).NE.' ') THEN
				OUTFIL=CMPFIL
				LOUTFL=LCMPFL
			ELSEIF (CMPUIC(1:1).EQ.' ') THEN
				OUTFIL=MECHID(1:LMECID)//FILDLM
     &				 //PROFIL(1:LPROFL)//'.CMP'
				LOUTFL=LMECID+LPROFL+5
			ELSE
				OUTFIL=CMPUIC(1:LCMPUC)//
     &				 MECHID(1:LMECID)//FILDLM
     &				 //PROFIL(1:LPROFL)//'.CMP'
				LOUTFL=LCMPUC+LMECID+LPROFL+5
			ENDIF
			OPEN (UNIT=3,FILE=OUTFIL,STATUS='UNKNOWN')
			WRITE (3,1) 'PROF='//PROFIL(1:8)
     &			 //' MECH='//MECHID(1:LMECID)
C
			TMSNC=0.0
			DO 2017 I=1,NMS
 2017			TMSNC=TMSNC+(MSMOL(I)*FLOAT(MSNC(I)))
C
			IF (CMPOPT.EQ.'M'.AND.TOTMAS.NE.0.0) THEN
				WRITE (3,1) '!MASS NORMALIZED'
				CMPFAC=1.0/TOTMAS
			ELSEIF (CMPOPT.EQ.'C'.AND.TMSNC.NE.0.0) THEN
				WRITE (3,1) '!CARBON NORMALIZED'
				CMPFAC=1.0/TMSNC
			ELSE
				CMPFAC=1.0
			ENDIF
			WRITE (3,2022) '!TOTMASS',TOTMAS*CMPFAC
			IF (UNKMAS.NE.0.0) THEN
			    IF (UNKIND.EQ.0) THEN
				WRITE (3,2022) '!UNKMASS',
     &				 UNKMAS*CMPFAC,' REP=NONE'
			    ELSEIF (UNKIND.EQ.-2) THEN
				WRITE (3,2022) '!UNKMASS',
     &				 UNKMAS*CMPFAC,' REP=INERT'
			    ELSEIF (UNKIND.EQ.-1) THEN
				WRITE (3,2022) '!UNKMASS',
     &				 UNKMAS*CMPFAC,' REP=NORM'
			    ELSE
				WRITE (3,2022) '!UNKMASS',
     &				 UNKMAS*CMPFAC,
     &				 ' REP='//UNKEMC(1:LNBLNK(UNKEMC))
			    ENDIF
			ENDIF
			WRITE (3,2022) '!TOTMOL ',TOTMOL*CMPFAC
			IF (UNAMOL.NE.0.0) THEN
			    IF (UNAIND.EQ.0) THEN
				WRITE (3,2022) '!UNAMOL ',
     &				 UNAMOL*CMPFAC,' REP=NONE'
			    ELSEIF (UNAIND.EQ.0) THEN
				WRITE (3,2022) '!UNAMOL ',
     &				 UNAMOL*CMPFAC,' REP=INERT'
			    ELSEIF (UNAIND.EQ.-1) THEN
				WRITE (3,2022) '!UNAMOL ',
     &				 UNAMOL*CMPFAC,' REP=NORM'
			    ELSE
				WRITE (3,2022) '!UNAMOL ',
     &				 UNAMOL*CMPFAC,
     &				 ' REP='//UNAEMC(1:LNBLNK(UNKEMC))
			    ENDIF
			ENDIF
			WRITE (3,2022) '!TOTNC  ',TOTNC*CMPFAC
			WRITE (3,2022) '!MODNC  ',TMSNC*CMPFAC
			IF (TMSNC.GT.0) WRITE (3,2022) '!WT/MODC',
     &			 TOTMAS/TMSNC
			WRITE (3,1) '!'
			DO 2014 I=1,NMS
			IF (MSMOL(I).NE.0.0) WRITE (3,2022) MSNAM(I),
     &			 MSMOL(I)*CMPFAC
 2014			CONTINUE
			CLOSE (UNIT=3)
			WRITE (6,*) 'File = '//OUTFIL(1:LOUTFL)//
     &			 ' written.'
		ENDIF
		LOSTC=LOSTC/TOTMOL
	ENDIF
  202	FORMAT (A14,9X,A8,6X,F16.11,F14.7,F10.7)
 2021	FORMAT (A,80(',',1PG12.4))
 2022	FORMAT (A8,1PE13.4,A)
C
C	MOLEC WT. PER CARBON
C
	IF (STDOUT) THEN
		IF (TOTNC.EQ.0.0) THEN
			CF=0.0
		ELSE
			CF=VOLMAS/TOTNC
		ENDIF
C
		WRITE (3,2031) PROCHR(1:LPROCH),TOTMAS,
     &		 VOLMAS,VOCMAS,AVGMW,TOTMOL,UNKMAS,UNAMAS,
     &		 UNAMOL,UNRMAS,NOVMAS,TOTNC,CF,LOSTC,LOSTM
2031		FORMAT(A,',ok',22(',',1PG13.6))
	ENDIF
C
	IF (VOCFAC.AND.(VOCMAS.GT.0.0)) THEN
		IF (MLTPRO) THEN
			WRITE (9,2033) PROFIL,TOTMAS/VOCMAS
 2033			FORMAT (A16,F8.4)
			
		ELSE
			WRITE (9,2032) PNUM,TOTMAS/VOCMAS
 2032			FORMAT (A8,F8.4)
		ENDIF
	ENDIF
C
	IF (ONEPRO) THEN
		CALL EXIT (0)
	ELSEIF (INU.EQ.0) THEN
		GOTO 800
	ELSE
		GOTO 100
	ENDIF
C
C	END OF INPUT.
C
  800	CLOSE (UNIT=1)
	IF (NPRO.EQ.0) THEN
		WRITE (6,*) 'No profiles processed for mechanism.'
	ELSE
		WRITE (6,*) NPRO,
     &		 ' profiles processed for mechanism.'
	ENDIF
	IF (.NOT.STDOUT) CALL EXIT (0)
C
	IF (MLTPRO) GOTO 900
C
C	-----------------------------------------------------------
C	MERGE FILES.  IGNORE PROFILES THAT WERE ALEREDY PROCESSED
C
	IFILE=1
 	INFILE=MRGFL1
	LINFIL=LMRGF1
 810	IF (OUTFOR.NE.1.OR.INFILE(1:1).EQ.' ') GOTO 900
	WRITE (6,*) 'Reading: '//INFILE(1:LINFIL)
	OPEN (UNIT=1,FILE=INFILE,STATUS='OLD',ERR=905)	! READONLY
	WRITE (3,1) 'GSPRO-IN,,,'//INFILE(1:LINFIL)
	PROIN='              '
	OUTOK=.FALSE.
C
  830	READ (1,1,END=840) INLIN
	IF (INLIN(1:1).EQ.'!'.OR.INLIN(1:4).EQ.'    ') GOTO 830
	IF (INLIN(1:14).EQ.PROIN) THEN
		IF (OUTOK) WRITE (2,1) INLIN(1:LNBLNK(INLIN))
		GOTO 830
	ENDIF
C
	PROIN=INLIN(1:14)
	PNUM=INLIN(1:4)
	PTYP=INLIN(7:14)
	LPNUM=LNBLNK(PNUM)
	LPTYP=LNBLNK(PTYP)
	PROCHR=PNUM(1:LPNUM)//','//PTYP(1:LPTYP)//','
	LPROCH=LPNUM+LPTYP+2
	DO 831 I=1,NPRO
 		IF (PROID(I).EQ.PROIN) THEN
			WRITE (3,1) PROCHR(1:LPROCH)//',Skipped'
 			OUTOK=.FALSE.
 			GOTO 830
 		ENDIF
  831	CONTINUE
	IF (NPRO.GT.MAXPRO) THEN
		WRITE (6,*) 'INCREASE MAXPRO IN SPECPRO.FOR'
		CALL EXIT (2)
	ENDIF
	NPRO=NPRO+1
	PROID(NPRO)=PROIN
	OUTOK=.TRUE.
	WRITE (3,1) PROCHR(1:LPROCH)//',Copied'
	WRITE (2,1) INLIN(1:LNBLNK(INLIN))
	GOTO 830
C
C	EOF ON MERGE FILE
C
  840	CLOSE (UNIT=1)
	IF (IFILE.EQ.1) THEN
		IFILE=2
		INFILE=MRGFL2
		LINFIL=LMRGF2
		GOTO 810
	ENDIF
C
C	DONE
C
  900	WRITE (6,*) 'New GSPRO File = '//GSPFIL(1:LGSPFL)
	WRITE (6,*) 'Number of profiles output = ',NPRO
	IF (XINFO) THEN
		CLOSE (UNIT=10)
		WRITE (6,*) 'File = SPECPRO1.CSV produced with '
     &		 //'additional info.'
	ENDIF
C
	WRITE (6,*) 'Processed profiles summarized in '//
     &		OUTFIL(1:LOUTFL)
	CLOSE (UNIT=3)
C
	IF (VOCFAC) THEN
		CLOSE (UNIT=9)
		WRITE (6,*) 'VOC-to-TOG factors given in '//
     &		 TOGVOC(1:LTOGVC)
	ENDIF
C
	CALL EXIT (0)
C
C	ERRORS
C
 9001	WRITE (6,*) 'Error opening input file= '//
     &	 PRMFIL(1:LNBLNK(PRMFIL))
	GOTO 999
 901	WRITE (6,*) 'Error opening profile list input file= '
     &	 //PROLFL(1:LPROLF)
	GOTO 999
 905	WRITE (6,*) 'Error opening GSPRO merge file = '
     &	 //INFILE(1:LINFIL)
	CALL EXIT (1)
C
 999	CALL EXIT (2)
	END
